using juobs, BDIO, DelimitedFiles, ADerrors, LaTeXStrings, PyPlot
using LaTeXStrings: length
using LsqFit

rcParams = PyPlot.PyDict(PyPlot.matplotlib."rcParams")
rcParams["text.usetex"] =  false
rcParams["mathtext.fontset"]  = "cm"
rcParams["font.size"] =10
rcParams["axes.labelsize"] =22
rcParams["axes.titlesize"] = 18
plt.rc("text", usetex=false) # set to true once you install latex


path_corr = "/Users/alessandroconigli/Lattice/data/charm_full_Dirac/N302/N302r001_correction_k0.137211_mup0.00292_mup0.00682_mup0.1320_mup0.1460.mesons.dat"
path_sl   = "/Users/alessandroconigli/Lattice/data/charm_full_Dirac/N302/N302r001_sloppy_k0.137211_mup0.00292_mup0.00682_mup0.1320_mup0.1460.mesons.dat"
path_rw   = "/Users/alessandroconigli/Lattice/data/aux_obs_data/rwf/N302/N302r001.ms1.dat"

ens = "N302"
##

wpmm = Dict{String, Vector{Float64}}()
wpmm["H101"]     = [-1.0, 2.0, -1.0, -1.0]
wpmm["H102r002"] = [-1.0, 2.0, -1.0, -1.0]
wpmm["H400"]     = [-1.0, 1.5, -1.0, -1.0]
wpmm["N202"]     = [-1.0, 2.0, -1.0, -1.0]
wpmm["N200"]     = [-1.0, 2.0, -1.0, -1.0]
wpmm["N203"]     = [-1.0, 2.0, -1.0, -1.0]
wpmm["N300"]     = [-1.0, 2.0, -1.0, -1.0]
wpmm["N302"]     = [-1.0, 2.0, -1.0, -1.0]
wpmm["J303"]     = [-1.0, 2.0, -1.0, -1.0]
wpmm["D200"] = [-1.0, 4.0, -1.0, -1.0]


##
# D200 param
# noise_trunc = [nothing, 1, 2, 4, 8]
# ncfg   = [2000, 2000, 1000, 500, 250]
# idmsl   = [collect(1:2000), collect(1:2000), collect(1:2:2000), collect(1:4:2000), collect(1:8:2000)]
noise_trunc = [16, 2, 4, 8]
ncfg   = [60, 60, 30, 15]
idmsl   = [collect(1:60), collect(1:60), collect(1:2:60), collect(1:4:60)]

## here i want to test, with full statistics, the TSM, sloppy, exact and exact-sloppy,corr observables
rwf = juobs.read_ms1(path_rw)
idm_corr= collect(1:20)
nms = 60
data_corr = juobs.read_mesons_correction(path_corr,"G5", "G5") # correction data combined
data_ex   = juobs.read_mesons(path_corr, "G5", "G5" )[1:9] # sloppy data only 
data_sl   = juobs.read_mesons(path_sl, "G5", "G5")    # exact data only 

pp_corr   = juobs.corr_obs.(data_corr, idm=idm_corr, nms=nms) # correction correlator
pp_ex     = juobs.corr_obs.(data_ex, idm=idm_corr, nms=nms) # exact correlator 
pp_sloppy = juobs.corr_obs.(data_sl, idm=idmsl[1], nms=nms) # sloppy correlator
pp_TSM    = juobs.corr_obs_TSM.(data_sl, data_corr, idm_corr=idm_corr, idm_sl=idmsl[1], nms=nms) # exact correlator

for midx in [1,4,5,6]
    TT = 75
    tmp_corr   = pp_corr[midx].obs[TT] ;   uwerr(tmp_corr, wpmm)
    tmp_ex     = pp_ex[midx].obs[TT] ;     uwerr(tmp_ex, wpmm)
    tmp_sloppy = pp_sloppy[midx].obs[TT] ; uwerr(tmp_sloppy, wpmm) 
    tmp_TSM    = pp_TSM[midx].obs[TT] ;    uwerr(tmp_TSM, wpmm)

    println("#############")
    println("Midx = $(midx)")
    println("TSM    = $(tmp_TSM)")
    println("Sloppy = $(tmp_sloppy)")
    println("Exact  = $(tmp_ex)")
    println("Correc = $(tmp_corr)")
    println("#############")


end

## here i am studing the tradeoff between noise sources and number of configurations.

ll = Vector()
pp = Vector{Vector{juobs.Corr}}(undef, 0)
for k in eachindex(noise_trunc)
    println(k)
    data_sl = juobs.read_mesons(path_sl, "G5", "G5", nnoise_trunc=noise_trunc[k])
    ppaux = juobs.corr_obs_TSM.(data_sl, data_corr, idm_corr=idm_corr, idm_sl=idmsl[1], nms=nms )
    push!(pp, ppaux )
    llaux = string("cnfg:",ncfg[k], " nnoise:", noise_trunc[k])
    push!(ll, llaux)
end

## pp corr vs cnfg and nnoise
PPfix = Vector{Vector{uwreal}}(undef, length(pp))
PPfixex = Vector{Vector{uwreal}}(undef, length(pp))

for k in eachindex(pp)
    PPfix[k] = Vector{uwreal}(undef, 0)
    PPfixex[k] = Vector{uwreal}(undef, 0)
    for midx in [1,4,5,6]
        yy = pp[k][midx].obs[75]
        yyex = pp_ex[midx].obs[75]
        push!(PPfix[k], yy)
        push!(PPfixex[k], yyex)

    end
end
tt = ["PPll", "PPlh", "PPsh", "PPhh"]
for k in eachindex(PPfix[1])
    fig = figure(figsize=(10,7.5))
    yy = getindex.(PPfix, k)
    yyex = getindex.(PPfixex, k)
    [uwerr(yy[ii], wpmm) for ii in eachindex(yy)]
    [uwerr(yyex[ii], wpmm) for ii in eachindex(yyex)]
    xx = collect(1:length(yy))
    errorbar(xx, value.(yy), err.(yy), fmt="*", capsize=2, label="TSM" )
    errorbar(xx.+0.1, value.(yyex), err.(yyex), fmt="*", capsize=2, label="Exact" )

    xticks(xx, string.(ll), rotation=15)
    title(tt[k])
    legend()
    display(fig)
    # savefig(joinpath("/Users/alessandroconigli/Lattice",string(tt[k],"_corrppNoiseCnfg.pdf")))
    close()
end

## print corr values at all nnoise and cnfg for all masses 

for k in eachindex(PPfix[1])
    yy = getindex.(PPfix, k)
    [uwerr(yy[ii], wpmm) for ii in eachindex(yy)]
    
    for elem in yy
        println(elem)
    end

    println("\n")
end





## meff vs cnfg and nnoise

M = Vector{Vector{uwreal}}(undef, length(pp))
for k in eachindex(pp)
    M[k] = Vector{uwreal}(undef, 0)
    cc = 1
    for midx in [1,4,5,6]

        if cc == 1
            tmin =collect(75:85)
            tmax = collect(105:115)
            tmax = tmax[1:2:end] 
            plat = [85,100]
        else
            tmin = cc==4 ? collect(86:88)  : collect(78:84)
            tmax = cc==4 ? collect(90:92)  : collect(95:100)
            plat = cc==4 ? [88,94]  : [85,95]

        end
        @. fitt(x,p) = p[1]
        maux = juobs.meff(pp[k][midx], plat, pl=true )
        # maux, syst, all_res, FitDict = juobs.bayesian_av(fitt, data, tmin, tmax, 2, pl=true, data=true, plt_title="ciao", path_plt=nothing, label=L"$m$")
        push!(M[k], maux)
        cc+=1
    end


end
##
# plot 

tt = ["mll", "mlh", "msh", "mhh"]
for k in eachindex(M[1])
    fig = figure(figsize=(10,7.5))
    yy = getindex.(M, k)
    xx = collect(1:length(yy))
    errorbar(xx, value.(yy), err.(yy), fmt="*", capsize=2 )

    xticks(xx, string.(ll), rotation=15)
    title(tt[k])
    display(fig)
    savefig(joinpath("/Users/alessandroconigli/Lattice",string(tt[k],"_testNoiseCnfg.pdf")))
    close()
end


##

idxm = 4
t0= meff(pp[1][idxm], [85,94])
t1 = meff(pp[2][idxm], [85,95])
t2 = meff(pp[3][idxm], [85,95])
t3 = meff(pp[4][idxm], [85,95])
t4 = meff(pp[5][idxm], [85,95])