###################################
##########      MASSS    ##########
###################################

##FUNCTIONS
basemodel(x,p) = p[1] .+ p[2] .* x[:,2] .+ p[3] .* x[:,3]  .+ p[4] .* x[:,1]
a2phi2(x)      = x[:,1] .* x[:,2]                  # phi2*a^2/8t0
a2phih2(x)     = x[:,1] .* x[:,3].^2               # phih^2*a^2/8t0          
a4phih4(x)     = x[:,1].^(2) .* x[:,3].^4          # phih^4*(a^2/8t0)^2 
a4phih4_2(x)   = x[:,1].^(2) .* x[:,3].^2          # (a^2/8t0)^2*phih^2 
a4cutoff(x)    = x[:,1].^(2)                       # (a^2/8t0)^2
a2loga(x)      = x[:,1] .* log.(x[:,1])     # (a^2/8t0)^2 * log(a^2/8t0)
phih2loga(x)   = x[:,3].^2 .* log.(x[:,1])  # phih^2*a^2/8t0 
# full list 
func_list = [a2phi2, a2phih2, a4phih4, a4phih4_2, a4cutoff]
label_list = ["a2l", "a2h2", "a4h4", "a4h2", "a4"]
func_map = [Bool.([i,j,k,m,n]) for i=0:1 for j=0:1 for k=0:1 for m=0:1 for n=0:1]

#  a^2log(a) and phih^2log(a)
# func_list = [ a2phih2, a2loga, phih2loga]
# label_list = [ "a2h2", "a2loga", "h2loga"]
# func_map = [Bool.([i,j,k]) for i=0:1 for j=0:1 for k=0:1  ]

# no a2l 
# func_list = [ a2phih2, a4phih4, a4phih4_2, a4cutoff]
# label_list = [ "a2h2", "a4h4", "a4h2", "a4"]
# func_map = [Bool.([i,j,k,m]) for i=0:1 for j=0:1 for k=0:1 for m=0:1]

##COMBINATIONS (LINEAR)
npar = length(func_list) # number of extra parameters 
n_param = Vector(4:4)
label_cutoff = Vector{Vector{String}}(undef, 0)
push!(label_cutoff, ["a2"])

#f_lin definition
f_lin = Vector{Vector{Function}}(undef, npar+1) #f[i][j]-> i: number of extra parameters, j: combinations
f_lin[1] = Vector{Function}(undef, 1)
f_lin[1][1] = (x,p) -> basemodel(x,p)

#f_aux definition -> similar to f_lin but without vectorized opeartors -> useful for plots
f_aux = Vector{Vector{Function}}(undef, npar+1) 
f_aux[1] = Vector{Function}(undef, 1)
f_aux[1][1] = (x,p) -> p[1] + p[2] * x[:,2] + p[3] * x[:,3] + p[4] * x[:,1]

#loop over combinations
for n = 2:npar+1
    aux = filter(x->sum(x) == n-1, func_map)
    f_lin[n] = Vector{Function}(undef, length(aux))
    f_aux[n] = Vector{Function}(undef, length(aux))
    k = 1
    for a in aux 
        #vectorized function 
        push!(n_param, 4+n-1)
        push!(label_cutoff, label_list[a])
        f_lin[n][k] = (x,p) -> basemodel(x,p) .+ sum([p[i+4] for i=1:(n-1)] .* (fill(x,n-1) .|> func_list[a]))
        f_aux[n][k] = (x,p) -> f_aux[1][1](x, p) + sum([p[i+4] for i=1:(n-1)] .* (fill(x,n-1) .|> func_list[a]))
        k = k + 1
    end
end

##COMBINATIONS (NONLINEAR)
f_non_lin = Vector{Vector{Function}}(undef, npar) #f[i][j]-> i: number of extra parameters, j: combinations
f_non_lin_aux = Vector{Vector{Function}}(undef, npar)
n_non_lin_param = Vector(5:npar+4) #param for each non-linear functions
for n = 1:npar
    aux = filter(x->sum(x) == n, func_map)
    f_non_lin[n] = Vector{Function}(undef, length(aux))
    f_non_lin_aux[n] = Vector{Function}(undef, length(aux))
    k = 1
    for a in aux 
        #vectorized function 
        push!(n_param, 4+n)
        f_non_lin[n][k] = (x,p) -> basemodel(x,p) .* (1 .+ sum([p[i+4] for i=1:(n)] .* (fill(x,n) .|> func_list[a])))
        f_non_lin_aux[n][k] = (x,p) -> p[1] + p[2] * x[:,2] + p[3] * x[:,3] + p[4] * x[:,1] +
        (p[1] + p[2] * x[:,2] + p[3] * x[:,3] + p[4] * x[:,1]) .* sum([p[i+4] for i=1:(n)] .* (fill(x,n) .|> func_list[a]))
        k = k + 1
    end
end

##APPEND LIN + NONLIN
f = f_lin
append!(f, f_non_lin)
append!(f_aux, f_non_lin_aux)


#n_param2 = vcat(n_param, n_non_lin_param)

########################################
##########     DEC. CONST.    ##########
########################################
# In the log functions I substituted p[4] with (1. .+ 3 .* 0.52)./(64. .* π.^2 .*  x[:,4]) to 
# test a fixed coefficient in front of the logs.
phi2_sym = 0.744666

##FUNCTIONS
f_a2phi2(x)    = x[:,1] .* x[:,2]           # phi2*a^2/8t0
f_a2phih2(x)   = x[:,1] .* (x[:,3]).^2      # phih^2*a^2/8t0          
f_a4phih4(x)   = x[:,1].^(2) .* x[:,3].^4   # phih^4*(a^2/8t0)^2          
f_a4phih2(x)   = x[:,1].^(2) .* x[:,3].^2   # phih^2*(a^2/8t0)^2  
f_a4cutoff(x)  = x[:,1].^(2)                # (a^2/8t0)^2
f_a2loga(x)    = x[:,1] .* log.(x[:,1])     # (a^2/8t0)^2 * log(a^2/8t0)
f_phih2loga(x) = x[:,3].^2 .* log.(x[:,1])  # phih^2*a^2/8t0 

##COMBINATIONS (LINEAR)
func_map = [Bool.([i,j,k, m, n]) for i=0:1 for j=0:1 for k=0:1 for m=0:1 for n=0:1]
f_basemodel1(x,p) = p[1] .+ p[2] .* x[:,2] .+ p[3] .* (1.0 ./ sqrt.(x[:,3])) .+ p[4] .* x[:,1]
f_basemodel2(x,p) = p[1] .+ p[2] .* (3 .* phi2_sym - 2 .* x[:,2]) .+ p[3] .* (1.0 ./ sqrt.(x[:,3])) .+ p[5] .* x[:,1]
#CHIRAL LOGS
f_basemodel1_clog(x,p) = p[1] .+ p[2] .* x[:,2] .+ p[3] .* (1.0 ./ sqrt.(x[:,3])) .+ p[5] .* x[:,1] .- (1. .+ 3 .* p[4])./(64. .* π.^2 .*  x[:,4]) .* (3 .* x[:,2] .*log.(x[:,2]) .+
 (3 .*phi2_sym .- x[:,2]) .* log.(1.5 .*phi2_sym .- 0.5*x[:,2]) .+ (2 .* phi2_sym .- x[:,2]) ./ 3 .* log.(2 .*phi2_sym .- x[:,2]))
f_basemodel2_clog(x,p) = p[1] .+ p[2] .* (3 .* phi2_sym - 2 .* x[:,2]) .+ p[3] .* (1.0 ./ sqrt.(x[:,3])) .+ p[6] .* x[:,1] .- 4 .* (1. .+ 3 .* p[4])./(64. .* π.^2 .*  x[:,4]) .*(
(1.5 .* phi2_sym .- 0.5.*x[:, 2]) .* log.(1.5 .* phi2_sym .- 0.5.*x[:, 2]) .+ (2 .* phi2_sym .- x[:,2]) ./ 3 .* log.((2 .* phi2_sym .- x[:,2])))

f_func_list = [f_a2phi2, f_a2phih2, f_a4phih4, f_a4phih2, f_a4cutoff]
#f_func_list = [f_a2loga, f_a2phih2, f_a4phih4, f_a4phih2, f_a4cutoff]

f_npar = length(f_func_list) # number of extra parameters 

## FLIN + FAUX
f_f_lin1 = Vector{Vector{Function}}(undef, f_npar+1) #f[i][j]-> i: number of extra parameters, j: combinations
f_f_lin2 = Vector{Vector{Function}}(undef, f_npar+1) #f[i][j]-> i: number of extra parameters, j: combinations
f_f_aux1 = Vector{Vector{Function}}(undef, f_npar+1)
f_f_aux2 = Vector{Vector{Function}}(undef, f_npar+1)

f_f_lin1[1] = Vector{Function}(undef, 1)
f_f_lin2[1] = Vector{Function}(undef, 1)
f_f_aux1[1] = Vector{Function}(undef, 1)
f_f_aux2[1] = Vector{Function}(undef, 1)

f_f_lin1[1][1] = (x,p) -> f_basemodel1(x, p)
f_f_lin2[1][1] = (x,p) -> f_basemodel2(x, p)

f_f_aux1[1][1] = (x,p) -> p[1] + p[2] * x[:,2] + p[3] * (1.0 ./ sqrt.(x[:,3])) + p[4] * x[:,1]
f_f_aux2[1][1] = (x,p) -> p[1] + p[2] * (3*phi2_sym - 2*x[:,2])  + p[3] * (1.0 ./ sqrt.(x[:,3])) + p[4] * x[:,1]

# FLIN + FAUX (CLOGS)  [hep-ph/9206230]
f_f_lin1_clog = Vector{Vector{Function}}(undef, f_npar+1) #f[i][j]-> i: number of extra parameters, j: combinations
f_f_lin2_clog = Vector{Vector{Function}}(undef, f_npar+1) #f[i][j]-> i: number of extra parameters, j: combinations
f_f_aux1_clog = Vector{Vector{Function}}(undef, f_npar+1)
f_f_aux2_clog = Vector{Vector{Function}}(undef, f_npar+1)

f_f_lin1_clog[1] = Vector{Function}(undef, 1)
f_f_lin2_clog[1] = Vector{Function}(undef, 1)
f_f_aux1_clog[1] = Vector{Function}(undef, 1)
f_f_aux2_clog[1] = Vector{Function}(undef, 1)

f_f_lin1_clog[1][1] = (x,p) -> f_basemodel1_clog(x, p)
f_f_lin2_clog[1][1] = (x,p) -> f_basemodel2_clog(x, p)

f_f_aux1_clog[1][1] = (x,p) -> p[1] + p[2] * x[:,2] + p[3] * (1.0 ./ sqrt.(x[:,3])) + p[5] * x[:,1] .- (1. .+ 3 .* p[4])./(64. .* π.^2 .*  x[:,4]) .* (3 * x[:,2] .*log.(x[:,2]) +
 (3 *phi2_sym - x[:,2]) .* log.(1.5 *phi2_sym - 0.5*x[:,2]) + (2 * phi2_sym - x[:,2]) / 3 .* log.(2 *phi2_sym - x[:,2]))
f_f_aux2_clog[1][1] = (x,p) -> p[1] + p[2] * (3 * phi2_sym - 2 * x[:,2]) + p[3] * (1.0 ./ sqrt.(x[:,3])) + p[6] * x[:,1] .- 4 .* (1. .+ 3 .* p[4])./(64. .* π.^2 .*  x[:,4]) .*(
(1.5 * phi2_sym - 0.5*x[:, 2]) .* log.(1.5 * phi2_sym - 0.5*x[:, 2]) + (2 * phi2_sym - x[:,2]) / 3 .* log.((2 * phi2_sym - x[:,2])))

## loop over combinations
for n = 2:f_npar+1
    aux = filter(x->sum(x) == n-1, func_map)
    f_f_lin1[n] = Vector{Function}(undef, length(aux))
    f_f_lin2[n] = Vector{Function}(undef, length(aux))
    f_f_aux1[n] = Vector{Function}(undef, length(aux))
    f_f_aux2[n] = Vector{Function}(undef, length(aux))
    f_f_lin1_clog[n] = Vector{Function}(undef, length(aux))
    f_f_lin2_clog[n] = Vector{Function}(undef, length(aux))
    f_f_aux1_clog[n] = Vector{Function}(undef, length(aux))
    f_f_aux2_clog[n] = Vector{Function}(undef, length(aux))
    k=1
    for a in aux 
        #vectorized function 
        f_f_lin1[n][k] = (x,p) -> f_basemodel1(x,p) .+ sum([p[i+5] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        f_f_aux1[n][k] = (x,p) -> f_f_aux1[1][1](x, p) +
        sum([p[i+5] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        f_f_lin2[n][k] = (x,p) -> f_basemodel2(x,p) .+ sum([p[i+6] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        f_f_aux2[n][k] = (x,p) -> f_f_aux2[1][1](x, p) +
        sum([p[i+6] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        
        f_f_lin1_clog[n][k] = (x,p) -> f_basemodel1_clog(x,p) .+ sum([p[i+6] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        f_f_aux1_clog[n][k] = (x,p) -> f_f_aux1_clog[1][1](x,p) +
        sum([p[i+6] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        f_f_lin2_clog[n][k] = (x,p) -> f_basemodel2_clog(x,p) .+ sum([p[i+7] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))
        f_f_aux2_clog[n][k] = (x,p) -> f_f_aux2_clog[1][1](x,p) +
        sum([p[i+7] for i=1:2:2*(n-1)] .* (fill(x,n-1) .|> f_func_list[a]))

        k = k+1
    end
end
f_npar_comb = 2 * f_npar
f_n_param_comb = collect(5:2:f_npar_comb+5)
f_n_param_comb_clog = collect(6:2:f_npar_comb+6)



########################################
########  RATIO DEC. CONST. ############
########################################
#  HQET and TAYLOR functional forms for (fDs*sqrt{mDs)}/(fD*sqrt{Md}) 
# from Gregorio's notes https://cloud.ift.uam-csic.es/s/9PGEbX5sDzcN5Xq#pdfviewer  


# base model from HQET
# before p[1] = \phi_f^2 in notes. Treated as a parameter but using fpk data would be better 
# now p[1]=x[:,5] fpik data from AS and p[2]=p[1]
r_basemodel_hqet(x,p) = 1. .- (1. .+ 3 .* p[3])./(64. .* π.^2 .*  x[:,5]) .*
    ((3 .*phi2_sym .- x[:,2]) .* log.(1.5 .* phi2_sym .- 0.5 .* x[:,2]) .+ #2L_k chiral log
     (2 .* phi2_sym .- x[:,2]) .* log.(2 .* phi2_sym .- x[:,2]) .- # L_\eta chiral log
      3 .* x[:,2] .* log.(x[:,2]) ) .+ # 3L_\pi chiral log 
     ((4. .* p[1]) ./ x[:,5]) .* (3 .* phi2_sym .- 3 .* x[:,2])

# higher order terms HQET   -  careful, explicit p[1]=\phi_f^2 dependece is discouraged  
r_chiphi2_hqet(x) = (4 ./ x[:,5].^2) .* (3 .* phi2_sym .- 3 .* x[:,2]) .* x[:,2]
r_chiphih_hqet(x) = (4 ./ x[:,5]) .* (3 .* phi2_sym .- 3 .* x[:,2]) ./ x[:,3]

# base model lattice spacing  dependece 
r_basemodel_continuum(x,p) = 1. .+ p[2] .* x[:,1] .* (3 .* phi2_sym .- 3 .* x[:,2])
# higher order lattice spacing  dependence 
r_phih_a_continuum(x) = x[:,1] .* (3 .* phi2_sym .- 3 .* x[:,2]) .* x[:,3].^2
r_cont_func_list = [r_phih_a_continuum]
r_cont_func_label = ["pa(3)"]
r_cont_extra_par = length(r_cont_func_list)
r_cont_func_map  = [Bool.([i]) for i=0:1]

r_hqet_func_list  = [r_chiphi2_hqet, r_chiphih_hqet ] #, r_phih_a_continuum] 
# r_hqet_func_label = [L"$p\chi^{(2)}$", L"$p_{\chi}^{(4)}$" ] #, L"$p_a^{(3)}$" ] 
r_hqet_func_label = ["p(2)", "p(4)" ] #, L"$p_a^{(3)}$" ] 
r_hqet_func_map  = [Bool.([i,j]) for i=0:1 for j=0:1 ]#for k in 0:1 ]


r_hqet_base_par = 3 # 2 from hqet base model, 1 from LO lattice spacing  dependence  
r_hqet_extra_par = length(r_hqet_func_list) # number of extra parameters on top of base model 

r_hqet_models = Vector{Function}(undef,1) #r_hqet_base_par+r_hqet_extra_par)
r_hqet_models[1] = (x,p) -> r_basemodel_hqet(x,p) .* r_basemodel_continuum(x,p) 
r_hqet_param = [3]
r_hqet_labels = [["hqet"]]

for extrapar = 1:r_hqet_extra_par
    idx_f_expar = filter(x->sum(x)==extrapar, r_hqet_func_map)

    # build and store  temporary hqet dependence 
    tmp_hqet_func  = Vector{Function}()
    tmp_hqet_param = []
    tmp_hqet_label = []
    for ii in idx_f_expar
        aux_f = (x,p) -> r_basemodel_hqet(x,p) .+ sum( p[r_hqet_base_par+k] .* r_hqet_func_list[ii][k](x) for k in eachindex(r_hqet_func_list[ii]))
        push!(tmp_hqet_param, r_hqet_base_par + extrapar)
        push!(tmp_hqet_func, aux_f)
        push!(tmp_hqet_label, r_hqet_func_label[ii])
    end
    # add continuum dependence - already code in view of adding more continuum higher order terms 
    for (jj, f_hqet) in enumerate(tmp_hqet_func)
        aux1 = (x,p) -> f_hqet(x,p) .* r_basemodel_continuum(x,p)
        push!(r_hqet_models, aux1)
        push!(r_hqet_param, tmp_hqet_param[jj])
        push!(r_hqet_labels, tmp_hqet_label[jj])

        for contpar = 1:r_cont_extra_par
            idx_cont_expar = filter(x->sum(x)==contpar, r_cont_func_map)
            for ii in idx_cont_expar
                aux_f_cont = (x,p) ->  f_hqet(x,p) .* (r_basemodel_continuum(x,p) .+ sum(p[r_hqet_base_par+extrapar+k] .* r_cont_func_list[ii][k](x) for k in eachindex(r_cont_func_list[ii]) ) ) 
                push!(r_hqet_models, aux_f_cont)
                push!(r_hqet_param, tmp_hqet_param[jj]+contpar)
                push!(r_hqet_labels, vcat(tmp_hqet_label[jj], r_cont_func_label[ii] ))
            end
        end
    end
end
# at the end of the above loop all the HQET combinations are taken into acount and store in r_hqet_models, r_hqet_param, r_hqet_labels
# now ration functional forms for Taylor based expressions 
# base model taylor 
r_basemodel_taylor(x,p) = 1. .+ 3 .*(phi2_sym .- x[:,2]) .*
    (p[1] .+ p[2] .* ( 1. ./ x[:,3] .- 1. ./ x[:,4]) )
#higher order models taylor 
r_taylor_nlo_phi2(x) = 3 .* (phi2_sym .- x[:,2]) .* x[:,2]
# this redefinition is required since in hqet LO parameter in continuum dependece is p[2] while here is p[3]
r_basemodel_continuum_taylor(x,p) = 1. .+ p[3] .* x[:,1] .* (3 .* phi2_sym .- 3 .* x[:,2])


r_taylor_func_list = [r_taylor_nlo_phi2]
r_taylor_func_label = ["pm(2)"]
r_taylor_func_map  = [Bool.([i]) for i=0:1]

r_taylor_base_par = 3 # 2 from taylor base model, 1 from LO lattice spacing dependence  
r_taylor_extra_par = length(r_taylor_func_list)

r_taylor_models = Vector{Function}(undef, 1)# r_basemodel_taylor + extra param
r_taylor_models[1] = (x,p) -> r_basemodel_taylor(x,p) .* r_basemodel_continuum_taylor(x,p)
r_taylor_param = [3]
r_taylor_labels = [["taylor"]]

for extrapar in 1:r_taylor_extra_par
    idx_f_expar = filter(x-> sum(x)==extrapar, r_taylor_func_map)
    # build and store temporary taylor dependence 
    tmp_taylor_func  = Vector{Function}()
    tmp_taylor_param = []
    tmp_taylor_label = []
    for ii in idx_f_expar
        aux_f = (x,p) -> r_basemodel_taylor(x,p) .+ sum(p[r_taylor_base_par+k] .* r_taylor_func_list[ii][k](x) for k in eachindex(r_taylor_func_list[ii]))
        push!(tmp_taylor_param, r_taylor_base_par +extrapar)
        push!(tmp_taylor_func, aux_f)
        push!(tmp_taylor_label, r_taylor_func_label[ii])
    end
    # add continuum dependence - already code in view of adding more continuum higher order terms 
    for (jj, f_taylor) in enumerate(tmp_taylor_func)
        aux1 = (x,p) -> f_taylor(x,p) .* r_basemodel_continuum_taylor(x,p)
        push!(r_taylor_models, aux1)
        push!(r_taylor_param, tmp_taylor_param[jj])
        push!(r_taylor_labels, tmp_taylor_label[jj])

        for contpar = 1:r_cont_extra_par
            idx_cont_expar = filter(x->sum(x)==contpar, r_cont_func_map)
            for ii in idx_cont_expar
                aux_f_cont = (x,p) -> f_taylor(x,p) .* (r_basemodel_continuum_taylor(x,p) .+ sum(p[r_taylor_base_par+extrapar+k] .* r_cont_func_list[ii][k](x) for k in eachindex(r_cont_func_list[ii]) ))
                push!(r_taylor_models, aux_f_cont)
                push!(r_taylor_param, tmp_taylor_param[jj]+contpar)
                push!(r_taylor_labels, vcat(tmp_taylor_label[jj], r_cont_func_label[ii]))
            end
        end
    end
end
# at the end this second loop all the TAYLOR combinations are taken into acount and store in r_taylor_models, r_taylor_param, r_taylor_labels

# final ration models are 
r_tot_models = vcat(r_hqet_models, r_taylor_models)
r_tot_param = vcat(r_hqet_param, r_taylor_param)
r_tot_labels = vcat(r_hqet_labels, r_taylor_labels)



########################################
########   DEC. CONST. HQET FORMS ############
########################################
#  HQET  functional forms for fDs*sqrt{mDs} and fD*sqrt{Md} 
# from Gregorio's notes https://cloud.ift.uam-csic.es/s/9PGEbX5sDzcN5Xq#pdfviewer  
# CONTINUUM DEPENDENCE 
phid_basemodel(x,p) = p[1] .- (1. .+ 3 .* p[2])./(64. .* π.^2 .*  x[:,4]) .* 
    (
        3 .* x[:,2] .* log.(x[:,2])  .+ # 3L_pi
        (3 .*phi2_sym .- x[:,2]) .* log.(1.5 .* phi2_sym .- 0.5 .* x[:,2]) .+ #2L_k
        1. ./3. .* (2 .* phi2_sym .- x[:,2]) .* log.(2 .* phi2_sym .- x[:,2])  # 1/3L_\eta
    ) .+ p[3] .* (4 .* x[:,2]) ./ (x[:,4])

phids_basemodel(x,p) = p[1] .- (1. .+ 3 .* p[2])./(64. .* π.^2 .*  x[:,4]) .* 
    (
        2 .* (3 .*phi2_sym .- x[:,2]) .* log.(1.5 .* phi2_sym .- 0.5 .* x[:,2]) .+ #2L_k
        4. ./3. .* (2 .* phi2_sym .- x[:,2]) .* log.(2 .* phi2_sym .- x[:,2])  # 1/3L_\eta
    ) .+  p[3] .* (4 .* (3 .* phi2_sym .- 2 .*x[:,2])) ./ ( x[:,4])

# higher order terms 
phid_chiphi2(x)  = (4 .* x[:,2].^2) ./ (x[:,4].^2)
phid_chiphih(x)  = (4 .+ x[:,2]) ./ (x[:,4] .* x[:,3])

phids_chiphi2(x) = (4 .* (3 .* phi2_sym .- 2 .*x[:,2]) .* x[:,2]) ./ ( x[:,4].^2)
phids_chiphih(x) = (4 .* (3 .* phi2_sym .- 2 .*x[:,2]) ) ./ ( x[:,4] .* x[:,3])

phid_ds_phih(x)    = 1 ./ x[:,3]   

d_hqet_func_list = [phid_chiphi2, phid_chiphih, phid_ds_phih]
ds_hqet_func_list = [phids_chiphi2, phids_chiphih, phid_ds_phih]
d_ds_hqet_func_label = ["p[chi](2)", "p[chi](4)", "ph(1)"]
d_ds_hqet_extra_param = length(d_hqet_func_list)
d_ds_hqet_func_map = [Bool.([i,j,k]) for i=0:1 for j=0:1 for k=0:1]

d_ds_base_par = 4 # 3 from cont dependence 1 from a^2 lattice spacing.

# LATTICE SPACING DEPENDENCE
phid_ds_base_cont(x,p) = 1. .+ p[4] .*  x[:,1]
#higher order terms
phid_cont_phi2(x) = x[:,1] .* x[:,2]
phids_cont_phi2(x) = x[:,1] .*  2. .*( 3 ./2 .* phi2_sym .-  x[:,2])

phid_cont_phi2phih(x) = x[:,1] .* x[:,2] .* x[:,3].^2
phids_cont_phi2phih(x) = x[:,1] .*  2. .* ( 3 ./2 .* phi2_sym .-  x[:,2]) .* x[:,3].^2

phid_ds_cont_phih(x) = x[:,1] .* x[:,3].^2

d_cont_func_list  = [phid_cont_phi2, phid_cont_phi2phih, phid_ds_cont_phih]
ds_cont_func_list = [phids_cont_phi2, phids_cont_phi2phih, phid_ds_cont_phih]
d_ds_cont_func_label = ["pa(1)", "ps(3)", "pa(2)"]
d_ds_cont_extra_param = length(d_cont_func_list)
d_ds_cont_func_map = [Bool.([i,j,k]) for i=0:1 for j=0:1 for k=0:1]


# building all functions 
phid_tot_models = Vector{Function}(undef, 1)
phid_tot_models[1] = (x,p) -> phid_basemodel(x,p) .* phid_ds_base_cont(x,p)

phids_tot_models = Vector{Function}(undef, 1)
phids_tot_models[1] = (x,p) -> phids_basemodel(x,p) .* phid_ds_base_cont(x,p)

d_ds_param = [4]           # n. of parameters for the leading order expression
d_ds_labels = [["hqet"]]     # label of the leading order expression

for extrapar = 1:d_ds_hqet_extra_param
    idx_f_expar = filter(x->sum(x)==extrapar, d_ds_hqet_func_map)

    tmp_fd_hqet  = Vector{Function}()
    tmp_fds_hqet = Vector{Function}()
    tmp_param = []
    tmp_label = []
    for ii in idx_f_expar
        aux_fd  = (x,p) -> phid_basemodel(x,p) .+ sum(p[d_ds_base_par+k] .* d_hqet_func_list[ii][k](x) for k in eachindex(d_hqet_func_list[ii]))
        aux_fds = (x,p) -> phids_basemodel(x,p) .+ sum(p[d_ds_base_par+k] .* ds_hqet_func_list[ii][k](x) for k in eachindex(ds_hqet_func_list[ii]))
        push!(tmp_fd_hqet, aux_fd)
        push!(tmp_fds_hqet, aux_fds)
        push!(tmp_param, d_ds_base_par + extrapar)
        push!(tmp_label, d_ds_hqet_func_label[ii])
    end
    # add lattice spacing dependence
    for  jj in eachindex(tmp_fd_hqet)
        aux_fd = tmp_fd_hqet[jj]
        aux_fds = tmp_fds_hqet[jj]

        aux1_fd  = (x,p) -> aux_fd(x,p) .* phid_ds_base_cont(x,p)
        aux1_fds = (x,p) -> aux_fds(x,p) .* phid_ds_base_cont(x,p) 
        push!(phid_tot_models, aux1_fd)
        push!(phids_tot_models, aux1_fds)
        push!(d_ds_labels, tmp_label[jj])
        push!(d_ds_param, tmp_param[jj])

        for contpar = 1:d_ds_cont_extra_param
            idx_cont_expar = filter(x->sum(x)==contpar, d_ds_cont_func_map)

            for ii in idx_cont_expar
                aux_fd_cont  = (x,p) -> aux_fd(x,p) .* (phid_ds_base_cont(x,p) .+ sum(p[d_ds_base_par+extrapar+k] .* d_cont_func_list[ii][k](x) for k in eachindex(d_cont_func_list[ii])))
                aux_fds_cont = (x,p) -> aux_fds(x,p) .* (phid_ds_base_cont(x,p) .+ sum(p[d_ds_base_par+extrapar+k] .* ds_cont_func_list[ii][k](x) for k in eachindex(ds_cont_func_list[ii])))
                push!(phid_tot_models, aux_fd_cont)
                push!(phids_tot_models, aux_fds_cont)
                push!(d_ds_labels, vcat(tmp_label[jj], d_ds_cont_func_label[ii]))
                push!(d_ds_param, tmp_param[jj]+contpar)
            end
        end
    end
end
# at the end of this loop all HQET + continuum combiations are taken into account for phid and phids and stored in 
# phid_tot_models and phids_tot_models respectively. Labels and parameters are given in d_ds_labels and d_ds_param














## 
# Operator overloading to call functions with uwreal parameters 
function Base.length(uw::uwreal)
    return length(value(uw))
end
# function Base.iterate(uw::uwreal)
    # uwerr(uw)
    # return iterate(value(uw), 1)
    # out = iterate(value(uw), args...; kwargs...)
    # return (uw, 1)
    # isnothing(out2) ?  (return nothing) : (return (uw, out2))  #(uw, out2) #(uw, getindex(out, 2))
    # return iterate(uw, args...)
    # return uwreal([iterate(value(uw), 1), err(uw)], " " )
# end
# function Base.iterate(uw::uwreal, i::Int64)
    # uwerr(uw)
    # return iterate(value(uw),i)
# end
function Base.iterate(uw::uwreal)
    return iterate(uw, 1)
end
function Base.iterate(uw::uwreal, state::Int64)
    if state > length(uw)
        return nothing
    else
        return uw[state], state +1
    end
end
function Base.iterate(uw::Vector{uwreal})
    return iterate(uw, 1)
end
function Base.iterate(uw::Vector{uwreal}, state::Int64)
    if state > length(uw)
        return nothing
    else
        return uw[state], state +1
    end
end
function Base.abs2(uw::uwreal)
    return (uw^2)^0.5
end
function Base.getindex(uw::uwreal, ii::Int64)
    idx = getindex(value(uw), ii)
    return  uw # uwreal([getindex(value(uw), kwargs...), err(uw)], " ")
end
# function Base.getindex(uw::Vector{uwreal}, ii::Int64)
    # idx = getindex(value.(uw), ii)
    # return  uw[ii] # uwreal([getindex(value(uw), kwargs...), err(uw)], " ")
# end

aaa = iterate(uwreal([5., 0.05], "c"))
aaa = iterate([[uwreal([5., 0.05], "c")]])