
using Base: String
using Base: @kwdef
using LaTeXStrings: length
using OrderedCollections
using juobs, BDIO, DelimitedFiles, ADerrors, LaTeXStrings, PyPlot
using ProgressMeter
#============= SET UP VARIABLES ===========#
rcParams = PyPlot.PyDict(PyPlot.matplotlib."rcParams")
rcParams["text.usetex"] =  true
rcParams["mathtext.fontset"]  = "cm"
rcParams["font.size"] =10
rcParams["axes.labelsize"] =22
rcParams["axes.titlesize"] = 18

plt.rc("text", usetex=true) # set to true once you install latex

#============ PATHS & INFO =================#
# path to data 
const path_data_w  = "/Users/alessandroconigli/Lattice/data/aux_obs_data/wilson" # required for t0 at finite a
const path_data_tm = "/Users/alessandroconigli/Lattice/data/charm_full_Dirac"    # charm correlators 
# path to plateau
const path_plat_w       = "/Users/alessandroconigli/Lattice/data/aux_obs_data/plat_wilson.txt"
const path_plat_tm_mps  = "/Users/alessandroconigli/Lattice/gevp-automated-analysis/plat/plat_tm_mps.txt"
const path_plat_tm_mvec = "/Users/alessandroconigli/Lattice/gevp-automated-analysis/plat/plat_tm_mvec.txt"
const path_plat_tm_dps  = "/Users/alessandroconigli/Lattice/gevp-automated-analysis/plat/plat_tm_dps.txt"
const path_plat_tm_dvec = "/Users/alessandroconigli/Lattice/gevp-automated-analysis/plat/plat_tm_dvec.txt"
# path to aux obs 
const path_rw = "/Users/alessandroconigli/Lattice/data/aux_obs_data/rwf"  # reweighting factors 
const path_md = "/Users/alessandroconigli/Lattice/data/aux_obs_data/md/"   # pbp dat file for mass shift 
# path to bdio
# const path_bdio_md         = "/Users/alessandroconigli/Lattice/data/bdio_charm/md/LAT22/" # path to delta m computed by me 
const path_bdio_md         = "/Users/alessandroconigli/Lattice/data/bdio_charm/AS_ligth_obs/" # path to delta m computed by AS
const path_bdio_tm         = "/Users/alessandroconigli/Lattice/data/bdio_charm/tm" # path where to store results 
const path_bdio_tm_shifted = "/Users/alessandroconigli/Lattice/data/bdio_charm/tm_shifted" # path were to store shifted results
const path_plot = "/Users/alessandroconigli/MyDrive/phd/analysis/charm_gevp_2022/CharmPaper/plots" # path were to save plots

# ensembles to analyse 
const ens_list = [ "J303"]
# reweighting and mass shift flags
const rwf = true
# println("Warning! mass_shift set to false ")
const mass_shift = false 
const tau = 3    # gevp shift parameter
const T0  = 2
const TSM = false # set whether TSM is used or not 

#@warning("\nTSM FLAG MODE: $(TSM)\n")

#=============== INCLUDES ===============#

include("types.jl")
include("tools.jl")
include("const.jl")
include("read_bdio.jl")

#=============== ENSEMBLE INFO FROM DATABASE ==================#
ensinfo = Vector{EnsInfo}(undef, length(ens_list))
for i in 1:length(ens_list)
    ens = ens_list[i]
    try
        ensinfo[i]= EnsInfo(ens, ens_db[ens], trunc_db[ens])
    catch
        error("The ensemble id ", ens, " was not found in the const.jl ens_db database.
        Please check the ensemble id or update the database")
    end
end
wpmm = Dict{String, Vector{Float64}}()
wpmm["H101"] = [-1.0, 2.0, -1.0, -1.0]  #14.0*2.86]
wpmm["H102r002"] = [-1.0, 2.0, -1.0, -1.0] #14.0*2.86]
wpmm["H105r002"] = [-1.0, 2.0, -1.0, -1.0]# 14.0*2.86]
wpmm["H400"] = [-1.0, 2.0, -1.0,  -1.0] #14.0*3.659]
wpmm["N202"] = [-1.0, 2.0, -1.0,  -1.0] #14.0*5.164]
wpmm["N200"] = [-1.0, 2.0, -1.0, -1.0]  #14.0*5.164]
wpmm["N203"] = [-1.0, 2.0, -1.0, -1.0] # 14.0*5.164]
#wpmm["D200"] = [-1.0, 4.0, -1.0, 14.0*5.164]
wpmm["D200"] = [-1.0, 2.0, -1.0, -1.0]
wpmm["N300r002"] = [-1.0, 2.0, -1.0, -1.0 ] #14.0*8.595]
wpmm["N302"] = [-1.0, 2.0, -1.0, -1.0 ] #14.0*8.595]
wpmm["J303"] = [-1.0, 2.0, -1.0, -1.0 ] #14.0*8.595]
wpmm["J500"] = [-1.0, 2.0, -1.0, -1.0 ] #14.0*8.595]
wpmm["J501"] = [-1.0, 2.0, -1.0, -1.0 ] #14.0*8.595]
wpmm["E250"] = [-1.0, 2.0, -1.0, -1.0 ] #14.0*8.595]

##
#=============== LOAD DATA AND STORE t0 ===============#
corr_pp  = Vector{Vector{juobs.Corr}}(undef, length(ens_list))
corr_pa  = Vector{Vector{juobs.Corr}}(undef, length(ens_list))
corr_aa  = Vector{Vector{juobs.Corr}}(undef, length(ens_list))
yw_store = Vector{Matrix{uwreal}}(undef, length(ens_list))          # store YW info from t0
w_store  = Vector{uwreal}(undef, length(ens_list))                  # store W info from t0
pp_tmW   = Vector{Vector{Vector{uwreal}}}(undef, length(ens_list))  # store W info from corr_pp
gen_mulist = Vector{Vector{Vector{Float64}}}(undef, length(ens_list)) # store mu values 

# obs storage  
obs_tm = Vector(undef, length(ensinfo))


@time begin
    @showprogress for (k,ens) in enumerate(ensinfo)
        println("\n Ensemble: ", ens.id)
        corr_pp[k], pp_tmW[k], _ =  get_corr(path_data_tm, ens, "G5", "G5", rw=rwf, path_rw=path_rw, tsm=TSM)
        gen_mulist[k] = getfield.(corr_pp[k], :mu)
    end
end
##
mlh, data_lh = juobs.meff(corr_pp[1][4], [127,150], data=true)
msh, data_sh = juobs.meff(corr_pp[1][5], [135,175], data=true)
mhh, data_hh = juobs.meff(corr_pp[1][6], [135,175], data=true)

data_lh ./= mlh ; uwerr.(data_lh)
data_sh ./= msh ; uwerr.(data_sh)
data_hh ./= mhh ; uwerr.(data_hh)

##
fig= figure("pyplot_subplot_column",figsize=(12,4))
subplots_adjust(wspace=0.0)
xx = collect(1:length(data_lh))

subplot(131)
errorbar(xx, value.(data_lh), err.(data_lh), fmt="*", capsize=2, color="navy", mfc="none", label=L"$D$")
fill_between(collect(127:150) , 1 -  err(mlh), 1 + err(mlh), color="red", alpha=0.5)
xlabel(L"$x_0/a$")
ylabel(L"$m^{\mathrm{eff}}(x_0)/m_{\mathrm{PS}}$")
xlim(96,190)
ylim(0.98,1.02)
legend()

aa = subplot(132)
setp(aa.get_yticklabels(), visible=false )
errorbar(xx, value.(data_sh), err.(data_sh), fmt="*", capsize=2, color="orange", mfc="none", label=L"$D_s$")
fill_between(collect(135:175) , 1 -  err(msh), 1 + err(msh), color="red", alpha=0.5)
xlabel(L"$x_0/a$")
xlim(96,190)
ylim(0.98,1.02)
legend()

aa=subplot(133)
setp(aa.get_yticklabels(), visible=false )
errorbar(xx, value.(data_hh), err.(data_hh), fmt="*", capsize=2, color="darkgreen", mfc="none", label=L"$\eta_c$")
fill_between(collect(135:175) , 1 -  err(mhh), 1 + err(mhh), color="red", alpha=0.5)
xlabel(L"$x_0/a$")
xlim(96,190)
ylim(0.98,1.02)
legend()

ax = fig.add_axes([0.78,0.3,0.15,0.2])
errorbar(xx, value.(data_hh), err.(data_hh), fmt="*", capsize=2, color="darkgreen", mfc="none")
fill_between(collect(135:175) , 1 -  err(mhh), 1 + err(mhh), color="red", alpha=0.5)
xlim(120,180)
ylim(0.999,1.001)

tight_layout()
display(gcf())
savefig("/Users/alessandroconigli/Desktop/meff_comparison.pdf")
close("all")