using Base: String
using Base: @kwdef
using LaTeXStrings: length
using OrderedCollections
using juobs, BDIO, DelimitedFiles, ADerrors, LaTeXStrings, PyPlot
using ProgressMeter
#============= SET UP VARIABLES ===========#
rcParams = PyPlot.PyDict(PyPlot.matplotlib."rcParams")
rcParams["text.usetex"] =  true
rcParams["mathtext.fontset"]  = "cm"
rcParams["font.size"] =10
rcParams["axes.labelsize"] =22
rcParams["axes.titlesize"] = 18

plt.rc("text", usetex=true) # set to true once you install latex

#============ PATHS & INFO =================#
# path to data 
const path_data_w  = "/Users/alessandroconigli/Lattice/data/aux_obs_data/wilson" # required for t0 at finite a
const path_data_tm = "/Users/alessandroconigli/Lattice/data/charm_full_Dirac"    # charm correlators 
# path to aux obs 
const path_rw = "/Users/alessandroconigli/Lattice/data/aux_obs_data/rwf"  # reweighting factors 
const path_md = "/Users/alessandroconigli/Lattice/data/aux_obs_data/md/"   # pbp dat file for mass shift 

const ens_list = [ "E250"]
# reweighting and mass shift flags
const rwf = false

#=============== INCLUDES ===============#

include("types.jl")
include("tools.jl")
include("const.jl")
include("read_bdio.jl")

ensinfo = Vector{EnsInfo}(undef, length(ens_list))
for i in 1:length(ens_list)
    ens = ens_list[i]
    try
        ensinfo[i]= EnsInfo(ens, ens_db[ens], trunc_db[ens])
    catch
        error("The ensemble id ", ens, " was not found in the const.jl ens_db database.
        Please check the ensemble id or update the database")
    end
end

##
# testing for E250 (in chunk2 and chunk2bis there's an overlap)
db = [ "sloppy_chunk0","sloppy_chunk1", "sloppy_chunk2", "sloppy_chunk3", "sloppy_chunk5", "sloppy_chunk5"] 
pp = joinpath(path_data_tm, "E250")
aux = filter(x-> occursin("sloppy_chunk", x), readdir(pp,join=true))

data_1 = juobs.read_mesons(aux[1], "G5","G5", legacy=false) # 1 -> 196
data_2 = juobs.read_mesons(aux[2], "G5","G5", legacy=false) # 201 -> 296 # trunc at 20 elem
data_2b = juobs.read_mesons(aux[3], "G5","G5", legacy=false) # 301 -> 396
data_3 = juobs.read_mesons(aux[4], "G5","G5", legacy=false) # 401 -> 596
data_4 = juobs.read_mesons(aux[5], "G5","G5", legacy=false) # 601 -> 796
data_5 = juobs.read_mesons(aux[6], "G5","G5", legacy=false) # 801 -> 1006



juobs.concat_data!(data_1, data_2)
juobs.concat_data!(data_1, data_2b)
juobs.concat_data!(data_1, data_3)
juobs.concat_data!(data_1, data_4)
juobs.concat_data!(data_1, data_5)


##

data_sl   = read_data_sloppy_multichunks(path_data_tm, "E250", "G5", "G5", legacy=false)
data_corr = read_data_correction(path_data_tm, "E250", "G5", "G5", legacy=false)

corr_pp, _ =  get_corr(path_data_tm, ensinfo[1], "G5", "G5", rw=false, path_rw=path_rw, tsm=true)

mm = meff.(corr_pp, fill([130, 150],9), pl=true )


function get_corr_TSM_multichunks(path::String, ens::EnsInfo, g1::String="G5", g2::String="G5"; rw::Bool=false, legacy::Bool=false, path_rw::String="", tsm::Bool=false, ex::Bool=false)
    if path_rw == ""
        p_rw = path
    else
        p_rw = path_rw
    end
    aux1 = read_data_sloppy_multichunks(path, ens.id, g1, g2, legacy=legacy)
    aux2 = read_data_correction(path, ens.id, g1, g2, legacy=legacy)
    try # single replica 
        # sloppy
        vcfg = getfield(aux1[1], :vcfg)
        delta = vcfg[1][2] - vcfg[1][1]
        cut_trunc = Int64.((ens.trunc .-1)./delta) .+ 1 
        truncate_data!(aux1, cut_trunc )
        # correction 
        vcfg = getfield(aux2[1], :vcfg)
        delta = vcfg[1][2] - vcfg[1][1]
        cut_trunc = Int64.((ens.trunc .-1)./delta) .+ 1
        truncate_data!(aux2, cut_trunc)
    catch # multiple replica
        # sloppy 
        vcfg = getfield.(aux1[1], :vcfg)
        delta = vcfg[1][2] - vcfg[1][1]
        cut_trunc = Int64.((ens.trunc .-1)./delta) .+ 1 
        truncate_data!(aux1, cut_trunc )
        # correction 
        vcfg = getfield.(aux2[1], :vcfg)
        delta = vcfg[1][2] - vcfg[1][1]
        cut_trunc = Int64.((ens.trunc .-1)./delta) .+ 1 
        truncate_data!(aux2, cut_trunc)
    end

    if !rw
        obs = corr_obs_TSM.(aux1, aux2, L=ens.L, info=true)#, idm_corr=collect(1:ens.idm_corr) )
        return (getindex.(obs, 1), getindex.(obs, 2))
    else
        obs = corr_obs_TSM.(aux1, aux2,  L=ens.L, rw=read_rw(p_rw, ens.id), info=true) #, idm_corr=collect(1:ens.idm_corr))
        return (getindex.(obs, 1), getindex.(obs, 2), getindex.(obs, 3))
    end

end

## testing apply_rw with gaps 


data_sl   = read_data_sloppy_multichunks(path_data_tm, "E300", "G5", "G5", legacy=false);
data_corr = read_data_correction(path_data_tm, "E300", "G5", "G5", legacy=false);


L = ensinfo[1].L;
idd = [ensinfo[1].id];
real_data = true;

vcfg_sl      = getfield(data_sl[1], :vcfg);
replica_sl   = Int64(maximum(vcfg_sl))
vcfg_corr    = getfield(data_corr[1], :vcfg);
replica_corr = Int64(maximum(vcfg_corr));

#truncate data

nms = sum(replica_sl);  # assuming vcfg_sl >= vcfg_corr

delta_vcfg = vcfg_sl[1][2] - vcfg_sl[1][1];
idm_sl = collect(1:delta_vcfg:replica_sl[1])
for ii in eachindex(data_sl[1])[2:end]
    delta_vcfg = vcfg_sl[ii][2] - vcfg_sl[ii][1]
    aux = collect(1:delta_vcfg:replica_sl[ii]) .+ replica_sl[ii-1]
    append!(idm_sl, aux)
end
delta_vcfg = vcfg_corr[1][2] - vcfg_corr[1][1]
idm_corr = collect(1:delta_vcfg:replica_corr[1])
for ii in eachindex(data_corr[1])[2:end]
    delta_vcfg = vcfg_corr[ii][2] - vcfg_corr[ii][1]
    aux = collect(1:delta_vcfg:replica_corr[ii]) .+ replica_corr[ii-1]
    append!(idm_corr, aux)
end


data1 = real_data ? getfield.(data_sl[1], :re_data) ./ L^3 : getfield.(data_sl, :im_data) ./ L^3
data2 = real_data ? getfield.(data_corr[1], :re_data) ./ L^3 : getfield.(data_corr, :im_data) ./ L^3

rw = read_rw(path_rw, ensinfo[1].id)
data1_r, W = juobs.apply_rw(data1, rw, vcfg_sl)
data2_r, W = juobs.apply_rw(data2, rw, vcfg_corr)

tmp1 = vcat(data1_r...)
tmp2 = vcat(data2_r...)
tmpW = vcat(W...)
nt = size(tmp1, 2)

ow1 = [uwreal(tmp1[:, x0], idd[1], replica_sl, idm_sl, nms ) for x0 = 1:nt]
ow2 = [uwreal(tmp2[:, x0], idd[1], replica_sl, idm_corr, nms) for x0 = 1:nt]

W_obs = uwreal(tmpW, idd[1], replica_sl, idm_sl, nms)

obs1 = [ow1[x0] / W_obs for x0 = 1:nt]
obs2 = [ow2[x0] / W_obs for x0 = 1:nt]

pp = juobs.Corr(obs1 + obs2, data_sl[1])