# read standard data no TSM

function read_data(path::String, ens::String, g1::String="G5", g2::String="G5"; legacy::Bool=false, ex::Bool=false)
    p = joinpath(path, ens)
    if !ex
        rep = filter(x->occursin("mesons.dat", x), readdir(p, join=true))
    else
        rep = filter(x->occursin("correction", x), readdir(p, join=true))
    end
    if length(rep)!=0    
        length(rep)==1 ? (return read_mesons(rep[1], g1, g2, legacy=legacy, id=ens)) : (return read_mesons(rep, g1, g2, legacy=legacy, id=ens))
    else
        error("mesons.dat file not found for ensemble ", ens, " in path ", p)
    end
end
# read sloppy TSM
function read_data_sloppy(path::String, ens::String, g1::String="G5", g2::String="G5"; legacy::Bool=false)
    p = joinpath(path, ens)
    rep = filter(x->occursin("sloppy", x), readdir(p, join=true))
    if length(rep)!=0    
        length(rep)==1 ? (return read_mesons(rep[1], g1, g2, legacy=legacy, id=ens)) : (return read_mesons(rep, g1, g2, legacy=legacy, id=ens))
    else
        error("mesons.dat file not found for ensemble ", ens, " in path ", p)
    end
end
function read_data_sloppy_multichunks(path::String, ens::String, g1::String="G5", g2::String="G5"; legacy::Bool=false)
    p = joinpath(path, ens)
    db = Dict("J500" => ["sloppy_chunk0", "sloppy_chunk1"], 
        "E300" => ["sloppy_chunk0", "sloppy_chunk1", "sloppy_chunk2" ],
        "N302" => ["sloppy"], "J501" => ["sloppy"],
        "E250" => [ "sloppy_chunk0","sloppy_chunk1", "sloppy_chunk2", "sloppy_chunk3", "sloppy_chunk4", "sloppy_chunk5"] 
    )
    store_cdata_aux = []
    for rep in db[ens]
        aux = filter(x-> occursin(rep, x), readdir(p,join=true))
        data_chunk = juobs.read_mesons(aux, g1, g2, legacy=legacy, id=ens) 
        if ens == "E250" && rep == "sloppy_chunk1"
            juobs.truncate_data!.(data_chunk, 20)
        end
        push!(store_cdata_aux, data_chunk)
    end
    for k in 2:length(store_cdata_aux)
        juobs.concat_data!(store_cdata_aux[1], store_cdata_aux[k])
    end

    if length(store_cdata_aux[1][1]) == 1
        return getindex.(store_cdata_aux[1], 1)
    else
        return store_cdata_aux[1]
    end
end
# read correction TSM
function read_data_correction(path::String, ens::String, g1::String="G5", g2::String="G5"; legacy::Bool=false)
    p = joinpath(path, ens)
    rep = filter(x->occursin("correction", x), readdir(p, join=true))
    if length(rep)!=0    
        length(rep)==1 ? (return read_mesons_correction(rep[1], g1, g2, legacy=legacy, id=ens)) : (return read_mesons_correction(rep, g1, g2, legacy=legacy, id=ens))
    else
        error("mesons.dat file not found for ensemble ", ens, " in path ", p)
    end
end
function read_pbp(path::String, ens::String)
    p = joinpath(path, ens)
    rep = filter(x->occursin("pbp", x), readdir(p, join=true))
    if length(rep)!=0
        if ens == "D200"
            md_aux = read_md.(rep)
            return [hcat(md_aux...)]
        end
        return read_md.(rep)
    else
        error("pbp.dat file not found for ensemble ", ens, " in path ", p)
    end
end
function read_rw(path::String, ens::String)
    p = joinpath(path, ens)
    rep = filter(x->occursin("ms1.dat", x), readdir(p, join=true))
    if ens == "J500"
        return [read_ms1(rep[1]), read_ms1(rep[2], v="1.4")]
    end
    if ens == "J501"
        return [read_ms1(rep[1]), read_ms1(rep[2], v="1.4") ]
    end
    if ens == "E250"
        return read_ms1(rep[1], v="1.6")
    end
    if length(rep)!=0
        try
            length(rep) == 1 ? (return read_ms1(rep[1])) : (return read_ms1.(rep)) 
        catch
            length(rep) == 1 && length(rep)!=0 ? (return read_ms1(rep[1], v="1.4")) : (return read_ms1.(rep, v="1.4")) 
        end
    else
        error("ms1.dat file not found for ensemble ", ens, " in path ", p)
    end
end
function read_t0(path::String, ens::String, dtr::Int64)
    p = joinpath(path, ens)
    rep = filter(x->occursin("ms.dat", x), readdir(p, join=true))
    if length(rep)!=0
        length(rep) == 1 ? (return read_ms(rep[1], dtr=dtr, id=ens)) : (return read_ms.(rep, dtr=dtr, id=ens)) 
    else
        error("ms.dat file not found for ensemble ", ens, " in path ", p)
    end
end
# in this function you have to implement the tsm reader 
function get_corr(path::String, ens::EnsInfo, g1::String="G5", g2::String="G5"; rw::Bool=false, legacy::Bool=false, path_rw::String="", tsm::Bool=false, ex::Bool=false)
    if path_rw == ""
        p_rw = path
    else
        p_rw = path_rw
    end
    if !tsm
        aux = read_data(path, ens.id, g1, g2, legacy=legacy, ex=ex)
        if !isnothing(ens.trunc)
            truncate_data!(aux, ens.trunc)
        end
        if !rw
            obs = corr_obs.(aux, L=ens.L, info=true)
            return (getindex.(obs, 1), getindex.(obs, 2))
        else
            obs = corr_obs.(aux, L=ens.L, rw=read_rw(p_rw, ens.id), info=true)
            return (getindex.(obs, 1), getindex.(obs, 2), getindex.(obs, 3))
        end
    else
        if ens.id in ["J500", "J501", "E250", "E300"]
            aux1 = read_data_sloppy_multichunks(path, ens.id, g1, g2, legacy=legacy)
        else
            aux1 = read_data_sloppy(path, ens.id, g1, g2, legacy=legacy)
        end
        aux2 = read_data_correction(path, ens.id, g1, g2, legacy=legacy)
        if !isnothing(ens.trunc)
            try # single replica 
                if ens.id == "E300"
                    truncate_data!(aux1, 455)
                else
                    vcfg = getfield(aux1[1], :vcfg)
                    delta = vcfg[2] - vcfg[1]
                    cut_trunc = Int64.((ens.trunc .-1)./delta) .+ 1 
                    println(cut_trunc)
                    truncate_data!(aux1, cut_trunc )
                    vcfg2 = getfield(aux2[1], :vcfg)
                    delta2 = vcfg2[2] - vcfg2[1]
                    cut_trunc2 = Int64.((ens.trunc .-1)./delta2) .+ 1 
                    truncate_data!(aux2, cut_trunc2)
                end
            catch # multiple replica
                vcfg = getfield.(aux1[1], :vcfg)
                delta = vcfg[1][2] - vcfg[1][1]
                cut_trunc = Int64.((ens.trunc .-1)./delta) .+ 1 
                truncate_data!(aux1, cut_trunc )
                vcfg2 = getfield.(aux2[1], :vcfg)
                delta2 = vcfg2[1][2] - vcfg2[1][1]
                cut_trunc2 = Int64.((ens.trunc .-1)./delta2) .+ 1 
                truncate_data!(aux2, cut_trunc2)
            end
            
        end
        if !rw
            obs = corr_obs_TSM.(aux1, aux2, L=ens.L, info=true)#, idm_corr=collect(1:ens.idm_corr) )
            return (getindex.(obs, 1), getindex.(obs, 2))
        else
            obs = corr_obs_TSM.(aux1, aux2,  L=ens.L, rw=read_rw(p_rw, ens.id), info=true) #, idm_corr=collect(1:ens.idm_corr))
            return (getindex.(obs, 1), getindex.(obs, 2), getindex.(obs, 3))
        end
    end
    
end
function get_t0(path::String, ens::EnsInfo; rw::Bool=false, pl::Bool=false, path_rw::String="")
    if path_rw == ""
        p_rw = path
    else
        p_rw = path_rw
    end
    data = read_t0(path, ens.id, ens.dtr)
    if !isnothing(ens.trunc)
        truncate_data!(data, ens.trunc)
    end
    plat = select_plateau_t0(ens) 
    if !rw
        t0 = comp_t0(data, plat, L=ens.L, pl=pl, info=true) 
        return (getindex(t0, 1), getindex(t0, 2))
    else
        t0 = comp_t0(data, plat, L=ens.L, pl=pl, rw=read_rw(p_rw, ens.id), info=true)
        return (getindex(t0, 1), getindex(t0, 2), getindex(t0, 3))
    end
end
function get_mu(mu_list::Vector{Vector{Float64}}, deg::Bool)
    mu_sorted = unique(sort(minimum.(mu_list)))

    mul = mu_sorted[1]
    deg ? mus = 0.0 : mus = mu_sorted[2]
    muh = unique(maximum.(mu_list))
    muh = filter(x-> x > mul && x > mus, muh)

    return mul, mus, muh
end
function get_kappa(kappa_list::Vector{Vector{Float64}}, deg::Bool)
    kappa_sorted = unique(sort(maximum.(kappa_list), rev=true))

    kappal = kappa_sorted[1]
    deg ? kappas = 0.0 : kappas = kappa_sorted[2]

    return kappal, kappas
end
function get_ll(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] == mu[2] #l-l
            return obs[k]
        end
    end
end
function get_ll_w(kappa_list, obs::Vector, deg::Bool)
    kappal, kappas = get_kappa(kappa_list, deg)
    for k = 1:length(kappa_list)
        kappa = kappa_list[k]
        if kappal in kappa && kappa[1] == kappa[2] #l-l
            return obs[k]
        end
    end
end
function get_ls(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mus in mu  #l-l
            return obs[k]
        end
    end
end
function get_ls_w(kappa_list, obs::Vector, deg::Bool)
    kappal, kappas = get_kappa(kappa_list, deg)
    for k = 1:length(kappa_list)
        kappa = kappa_list[k]
        if kappal in kappa && kappas in kappa  #l-l
            return obs[k]
        end
    end
end
function get_lh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_lh = Vector{uwreal}(undef, 0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] != mu[2] && !(mus in mu) #l-h
            push!(obs_lh, obs[k])
        end
    end
    return obs_lh
end
function get_ss(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mus in mu && mu[1] == mu[2] #s-s
            return obs[k]
        end
    end
end
function get_sh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_sh = Vector{uwreal}(undef, 0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mus in mu && mu[1] != mu[2] && !(mul in mu)#s-h
            push!(obs_sh, obs[k])
        end
    end
    return obs_sh
end

function get_hh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_hh = Vector{uwreal}(undef,0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        for i =1:length(muh)
            if muh[i] in mu && mu[1] == mu[2]#h-h
                push!(obs_hh, obs[k])
            end
        end
    end
    return obs_hh
end
"""
comp_mat_tau_shift(ensinfo::EnsInfo, tot_obs::Vector{juobs.Corr}, tau::Int64)

This method takes as input an EnsInfo object and a vector of Corr. For each element of the vector a 2x2 matrix is computed where each element of the matrix is shifted by a tau=4 value in time.
It is used for gevp problem with single gamma structure and a tau shift in time.
The method return a vector of MatInfo for each element of the vector Corr, corresponding to different masses

"""
function comp_mat_tau_shift(ensinfo::EnsInfo, tot_obs::Vector{juobs.Corr}, tau::Int64)
    mat_info = Vector{MatInfo}(undef, length(tot_obs))
    y0 = getfield.(tot_obs,:y0) .+1

    for i = 1:length(tot_obs)
        a11 = tot_obs[i].obs[y0[i]:end-1-2*tau]
        a12 = tot_obs[i].obs[y0[i]+tau:end-1-tau]
        a22 = tot_obs[i].obs[y0[i]+2*tau:end-1]
        aux = juobs.get_matrix([a11, a22], [a12])
        mat_info[i] = MatInfo(ensinfo, aux, tot_obs[i].mu, tot_obs[i].y0)
    end
    return mat_info
end
"""
comp_mat_multigamma(ensinfo::EnsInfo,tot11::Vector{juobs.Corr}, tot12::Vector{juobs.Corr}, tot22::Vector{juobs.Corr})
This method takes as input an EnsInfo object and:
tot11::Vector{juobs.Corr} corresponding to element 11
tot12::Vector{juobs.Corr} corresponding to element 12
tot22::Vector{juobs.Corr} corresponding to element 22

For each element of these vector a MatInfo object is built running on all the timeslices.
This method is used for gevp problems with multiple gamma structures in the matrix.
It returns a vector of MatInfo.
"""
function comp_mat_multigamma(ensinfo::EnsInfo,tot11::Vector{juobs.Corr}, tot12::Vector{juobs.Corr}, tot22::Vector{juobs.Corr})
    mu = getfield.(tot11,:mu)
    y0 = getfield.(tot11,:y0) 

    mat_info = Vector{MatInfo}(undef, length(mu))
    for i = 1:length(mu)
        a11 = tot11[i].obs[y0[i]:end-1]
        a12 = tot12[i].obs[y0[i]:end-1]
        a22 = tot22[i].obs[y0[i]:end-1]
        aux = juobs.get_matrix([a11, a22], [a12])
        mat_info[i] = MatInfo(ensinfo, aux, mu[i], y0[i])
    end
    return mat_info
end
"""
comp_mat_GPOF(ensinfo::EnsInfo, corr::Vector{juobs.Corr}; N::Int64=4)
This method takes as input an EnsInfo object and a vector of Corr.
For each element of the vector, a list of matrix of dimension NxN is built,
where each element of the matrix is shifted in time by one unit with respect to the previous element. 
This method is used to solve gevp a la BMW, i.e. by solving a single gevp with fixed ta and tb and then using the other matrices and the eigenvectors 
to project the original correlator into the ground state.
It returns a vector of MatInfo for each element of the vector of Corr.
"""
function comp_mat_GPOF(ensinfo::EnsInfo, corr::Vector{juobs.Corr}; N::Int64=4)
    mat_info = Vector{MatInfo}(undef, length(corr))
    y0 = getfield(corr[1], :y0) + 1 
    mu = getfield(corr[1], :mu)


    for i in 1:length(corr)
        tmp = Vector{Matrix}(undef, 0)
        obs = getfield(corr[i], :obs)
        
        for t in y0:length(obs)-10
            tmp1 = Matrix(undef, N,N)
            for row in 1:N
                for col in 1:N
                    tmp1[row,col] = obs[t+row+col-2]
                end
            end
            push!(tmp, tmp1)
        end
        mat_info[i] = MatInfo(ensinfo, tmp, mu, y0)        
    end
    return mat_info

end
function comp_mat_multimass(ensinfo::EnsInfo, pp::Vector{juobs.Corr}, pa0::Vector{juobs.Corr}  )
    mu = vcat(getfield.(pp,:mu)...)
    y0  = getfield(pp[1], :y0) +1 
    ppobs = getfield.(pp, :obs)
    pa0obs = getfield.(pa0, :obs)
    mat_info = Vector{MatInfo}(undef, length(mu))
    for i in eachindex(ppobs)
        aux = juobs.get_matrix([ppobs[1], ppobs[2],ppobs[3]], [pa0obs[1], pa0obs[2],pa0obs[3]] )
        mat_info[i] = MatInfo(ensinfo, aux, mu[i], y0)
    end
    return mat_info
end

function gevp_mass(mat_obs::Vector{MatInfo}, path_plat::String; t0::Int64=2,pl::Bool=true, wpm::Union{Nothing, Dict{String,Vector{Float64}}}=nothing)
    y0s = getfield.(mat_obs, :y0) 
    mu_list = getfield.(mat_obs, :mu) 
    plat_en0 = Vector{uwreal}(undef, length(mat_obs))
    plat_en1 = Vector{uwreal}(undef, length(mat_obs))
    for i in 1:length(mat_obs)
        mat = getfield(mat_obs[i], :mat_list)[y0s[i]+1:end-1] #matrices to diagonalise for a given sector [i]
        evals = getall_eigvals(mat, t0)
        en = energies(evals, wpm=wpm) #ground and excited state energy
        plat = select_plateau(mat_obs[i].ensinfo, mu_list, path_plat)[i] .- y0s[i] 
        println(plat)
        plat_en0[i] = plat_av(en[1], plat, wpm)
        if pl 
            uwerr(plat_en0[i])
            xx = collect(1:length(en[1]))
            v = value(plat_en0[i])
            e = err(plat_en0[i])
            errorbar(xx, value.(en[1]), err.(en[1]), fmt="s")
            fill_between(plat[1]:plat[2], v-e, v+e )
            ylim(e*0.9, e*1.1)
            display(gcf())
            close()
        end
    end
    return plat_en0
end

function gevp_mass(mat_obs::Vector{MatInfo}, path_m_ps::String, path_m_vec::String; t0::Int64=2,pl::Bool=true, wpm::Union{Nothing, Dict{String,Vector{Float64}}}=nothing)
    y0s = getfield.(mat_obs, :y0) 
    mu_list = getfield.(mat_obs, :mu) 
    plat_en0 = Vector{uwreal}(undef, length(mat_obs))
    plat_en1 = Vector{uwreal}(undef, length(mat_obs))
    for i in 1:length(mat_obs)
        mat = getfield(mat_obs[i], :mat_list) #[y0s[i]+1:end-1] #matrices to diagonalise for a given sector [i]
        evals = getall_eigvals(mat, t0)
        en = energies(evals, wpm=wpm) #ground and excited state energy
        plat_ps = select_plateau(mat_obs[i].ensinfo, mu_list, path_m_ps)[i] .- y0s[i] 
        println(plat_ps)
        plat_vec = select_plateau(mat_obs[i].ensinfo, mu_list, path_m_vec)[i] .- y0s[i] 
        println(plat_vec)
        plat_en0[i] = plat_av(en[1], plat_ps, wpm)
        # plat_en1[i] = plat_av(en[2], plat_vec, wpm)
        if pl 
            fig = figure(figsize=(10,6))
            isnothing(wpm) ? uwerr(plat_en0[i]) : uwerr(plat_en0[i],wpm)
            # isnothing(wpm) ? uwerr(plat_en1[i]) : uwerr(plat_en1[i],wpm)
            xx    = collect(y0s[1]:length(en[1])+y0s[1]-1)
            println(length(en[1]))
            v     = value(plat_en0[i])
            e     = err(plat_en0[i])
            # v_vec = value(plat_en1[i])
            # e_vec = err(plat_en1[i])
            errorbar(xx, value.(en[1]), err.(en[1]), fmt=">", mfc="none", capsize=2, ms=12, label=L"$M_{D_{(s)}}$")
            fill_between(plat_ps[1]+y0s[1]:plat_ps[2]+y0s[1], v-e, v+e, alpha=0.6 )

            # errorbar(xx[1:end-10], value.(en[2])[1:end-10], err.(en[2][1:end-10]), fmt="^",mfc="none", ms=12, capsize=2, label=L"$M_{D_{(s)}}^*$")
            # fill_between(plat_vec[1]+y0s[1]:plat_vec[2]+y0s[1], v_vec-e_vec, v_vec+e_vec, alpha=0.6 )
            xlabel(L"$x_0/a$")
            ylabel(L"$am_{\mathrm{eff}}$")
            title("$(ensembles(plat_en0[i])[1]) " * L"\mu_1="*"$(mu_list[i][1]) " *L" \mu_2="*"$(mu_list[i][2]) " )
            # xlim(65, 110)
            ylim(v-25*e, v+25*e)
            # legend(fontsize=20)
            display(gcf())
            t = "$(ensembles(plat_en0[i])[1])_m_eff_"*"mu_1=$(mu_list[i][1]) " *"_mu_2=$(mu_list[i][2]) "*".pdf"
            #savefig(joinpath(path_plot,t))
            close()
        end
    end
    return plat_en0, plat_en1
end
"""
gevp_mass_BMA(mat_obs::Vector{MatInfo}; t0::Int64=2,pl::Bool=true, wpm::Union{Nothing, Dict{String,Vector{Float64}}}=nothing)

This function takes as input a vector of MatInfo and for each of them solve the associated  eigenvalue problem.
Then, the  ground state energies extracted from the eigenvalues are fitted varying several time ranges and a Bayesian Model Average 
is performed to asses a final result. 
It can be used with both MatInfo with different gamma structure (comp_mat_multigamma) and with MatInfo with time shifts (comp_mat_tau_shift)
The method returns the weighted average of the ground state energy for each element of the vector of MatInfo.
"""
function gevp_mass_BMA(mat_obs::Vector{MatInfo}; tt0::Int64=2,pl::Bool=true, wpm::Union{Nothing, Dict{String,Vector{Float64}}}=nothing, path_plt::Union{String,Nothing}=nothing)
    y0s     = getfield.(mat_obs, :y0) 
    mu_list = getfield.(mat_obs, :mu) 
    en0     = Vector{uwreal}(undef, length(mat_obs))
    en1     = Vector{uwreal}(undef, length(mat_obs))
    data_tot = Vector(undef, length(mat_obs))
    W_tot = Vector(undef, length(mat_obs))

    @showprogress for k in eachindex(mat_obs)
        t = string(L"$\mu_1 = $", mu_list[k][1],L" $\mu_2 = $", mu_list[k][2] )   
        mat = getfield(mat_obs[k], :mat_list) #[y0s[i]+1:end-1] #matrices to diagonalise for a given sector [i]
        evals = juobs.getall_eigvals(mat, tt0)
        # evecs = getall_eig(mat, t0)
        entot = juobs.energies(evals, wpm=wpm)

        tminaux = round.(Int64, collect(18:38) *  sqrt(value(t0(mat_obs[k].ensinfo.beta) / t0(3.7))))
        T = length(entot[1])
        tmaxaux = round.(Int64, collect(10:24) *  sqrt(value(t0(mat_obs[k].ensinfo.beta) / t0(3.7))))
        tmax = T .- collect(reverse(tmaxaux))
        tmax = tmax[1:3:end]
        tmin = tminaux[1:2:end]
        if !isnothing(path_plt)
            pp1 = joinpath(path_plt, "plateaus", mat_obs[1].ensinfo.id, "mps", "mps_BMA_$(k).pdf")
            pp2 = joinpath(path_plt, "plateaus", mat_obs[1].ensinfo.id, "mvec", "mvec_BMA_$(k).pdf")

        else
            pp1 = pp2 = nothing
        end

        if mat_obs[1].ensinfo.id == "D200"
            @. gs_model(x,p) = p[1]  #+ p[2] * exp(-(p[3])*x)
            param = 1
            en0[k], syst1, all_res1, FitDict1 = juobs.bayesian_av(gs_model, entot[1], tmin, tmax, param, pl=pl, wpm=wpm, data=true, label=L"$m_{\mathrm{ps}}$", path_plt=pp1, plt_title=t)
            try
                en1[k], syst2, all_res2, FitDict2 = juobs.bayesian_av(gs_model, entot[2], tmin, tmax, param, pl=pl, wpm=wpm, data=true, label=L"$m_{\mathrm{v}}$", path_plt=pp2, plt_title=t)
            catch
                @warn "Vector mass failed"
                en1[k] = uwreal(1.0)
            end
        else
            @. gs_model1(x,p) = p[1]  #+ p[2] * exp(-(p[3])*x)
            param = 1
            en0[k], syst1, all_res1, FitDict1 = juobs.bayesian_av(gs_model1, entot[1], tmin, tmax, param, pl=pl, wpm=wpm, data=true, label=L"$m_{\mathrm{ps}}$", path_plt=pp1, plt_title=t)
            try
                en1[k], syst2, all_res2, FitDict2 = juobs.bayesian_av(gs_model1, entot[2], tmin, tmax, param, pl=pl, wpm=wpm, data=true, label=L"$m_{\mathrm{v}}$", path_plt=pp2, plt_title=t)
            catch
                @warn "Vector mass failed"
                en1[k] = uwreal(1.0)
            end
            
        end
        
        close("all")
        if pl 
            for ii in 1:2 # loop over energies
                try 
                    if ii == 1
                        data = entot[1]
                        FitDict = FitDict1
                        all_res = all_res1
                        if !isnothing(path_plt)
                            path_plt_fin = joinpath(path_plt, "plateaus", mat_obs[1].ensinfo.id, "mps")
                        end
                        ll =  L"$m_{\mathrm{ps}}$" 
                    else
                        data = entot[2]
                        FitDict = FitDict2
                        all_res = all_res2
                        if !isnothing(path_plt)
                            path_plt_fin = joinpath(path_plt, "plateaus", mat_obs[1].ensinfo.id, "mvec")
                        end
                        ll =  L"$m_{\mathrm{v}}$" 
                    end
                    fig = figure(figsize=(10,7.5))
                    modstot = FitDict["mods"]
                    AIC  = FitDict["AIC"]
                    Wtot = FitDict["W"]
                    IDX  = partialsortperm(AIC, 1:3)
                    mods = modstot[IDX]
                    main_res = all_res[IDX]
                    W = Wtot[IDX]
                    isnothing(wpm) ? uwerr(data) : [uwerr(data[i],wpm) for i in eachindex(data)]
                    vv = value.(data)[1:end]
                    ee = err.(data)[1:end]
                    xx = collect(0:length(vv)-1) 
                    errorbar(xx, vv, ee, fmt="d", mfc="none", color="black", capsize=2)
                    CC = ["forestgreen", "royalblue", "tomato"]
                    for (ii,trange) in enumerate(mods)
                        isnothing(wpm) ? uwerr(main_res[ii]) : uwerr(main_res[ii], wpm)
                        vaux = value(main_res[ii])
                        eaux = err(main_res[ii])
                        regex = r"([0-9]+),([0-9]+)"
                        aux = match(regex, trange)
                        fill_between(parse(Float64,aux[1]):parse(Float64,aux[2]), vaux-eaux, vaux+eaux, color=CC[ii], alpha=0.5, label=round(W[ii], digits=3))
                    end
                    axhline(y=value(main_res[1])-14*err(main_res[1]), xmin=(tmin[1])/length(xx), xmax=(tmin[end])/length(xx), ls="dashed", color="green")
                    axhline(y=value(main_res[1])-14.3*err(main_res[1]), xmin=(tmax[1])/length(xx), xmax=(tmax[end])/length(xx), ls="dashed", color="red")
                    text(tmin[1],value(main_res[1])-13*err(main_res[1]), L"$\{t_{\mathrm{min}}\}$" )
                    text(tmax[1],value(main_res[1])-13*err(main_res[1]), L"$\{t_{\mathrm{max}}\}$" )
                    # axvline(tmin[1], ls="dashed", color="green")
                    # axvline(tmax[end], ls="dashed", color="red")
                    
                    legend(title="Weights")
                    title(t)
                    xlabel(L"$t/a$")
                    ylabel(ll)
                    xlim(xx[1], xx[end])
                    ylim(value(main_res[1]) -30*err(main_res[1]), value(main_res[1]) +30*err(main_res[1]))
                    twiny()
                    fm_time = round.((xx )[1:8:end] .* value(a(mat_obs[k].ensinfo.beta)), digits=3)
                    xticks(xx[1:8:end], fm_time)
                    # xticks()
                    xlabel(L"$t-t_0\ [\mathrm{fm}]$", fontsize=14)
                    display(fig)
                    if !isnothing(path_plt)
                        savefig(joinpath(path_plt_fin, "m$(k)_plateau.pdf"))
                    end
                    close()
                catch
                    println("plot failed")
                end
            end
        end
    end
    return en0, en1 #, data_tot, W_tot
end
""" 
gevp_GPOF(ensinfo::EnsInfo, corr::Vector{juobs.Corr}; N::Int64=4, TA::Vector{Int64}=[2,4], TB::Vector{Int64}=[7,8])
This method takes as input an EnsInfo object and a vector of Correlators. 
Then it calls comp_mat_GPOF to compute a matrix a la BMW. For each TA, TB, a gevp is solved and the corresponding eigenvectors are used to project the original correlators
to the ground state.
This function return a vector of vector of  uwreal, where the first index correspond to the index in mass of the original corr vector passed as input,
while the second index labels different choices of ta and tb for the GEVP.
"""
function gevp_GPOF(ensinfo::EnsInfo, corr::Vector{juobs.Corr}; N::Int64=4, TA::Vector{Int64}=[2,4], TB::Vector{Int64}=[7,8])
    mat_info = comp_mat_GPOF(ensinfo, corr, N=N)
    y0 = getfield(corr[1], :y0)
    mu = getfield(corr[1], :mu)
    new_corr = Vector{Vector{Vector{uwreal}}}(undef, length(mat_info))

    for i in eachindex(mat_info)
        mat = getfield(mat_info[i], :mat_list)
        new_corr[i] = Vector{Vector{uwreal}}(undef, 0)
        for ta in TA
            for tb in TB
                evalues, evecs = juobs.uweigen(mat[ta], mat[tb], iter = 20)
                aux_corr = [uwdot(juobs.transpose(evecs[:,N]), uwdot( mat[k], evecs[:,N]) )[1] for k in eachindex(mat)]
                push!(new_corr[i], aux_corr)
            end 
        end 
    end
    return new_corr
end
"""
gevp_GPOF_BMA(ensinfo::EnsInfo, corr::Vector{juobs.Corr}; N::Int64=4, TA::Vector{Int64}=[2], TB::Vector{Int64}=[7,8], pl::Bool=true, wpm::Union{Nothing, Dict{String,Vector{Float64}}}=nothing)
This function takes as input an EnsInfo object and a vector of Corr.
A projection to the ground state using gevp_GPOF is built for each element of corr. Then the new_corr is used to compute effective maasses
and finally the latter are fitted several times by varying the time ranges. Eventually, a BMA is performed (bayesian_av) for each value of the mass.
It returns a vector of uwreal containing the weighted averaged results for each value of the mass. 
"""
function gevp_GPOF_BMA(ensinfo::EnsInfo, corr::Vector{juobs.Corr}; N::Int64=4, TA::Vector{Int64}=[2], TB::Vector{Int64}=[7,8], pl::Bool=true, wpm::Union{Nothing, Dict{String,Vector{Float64}}}=nothing)
    new_corr = gevp_GPOF(ensinfo, corr, N=N, TA=TA, TB=TB)
    @. one_exp_fit(x,p) = p[1] + p[2]*exp(-p[3]*x)
    Nparam=3
    y0 = getfield(corr[1], :y0)
    T = length(corr[1].obs) - y0
    #  fit time ranges for bayesian_av
    tmin = collect(14:16)
    tmax = collect(T-35:T-34)

    MAve = Vector{uwreal}(undef, length(corr))


    for k in eachindex(new_corr) # mass index 
        AIC = Vector{Float64}(undef, 0)
        Masses = Vector{uwreal}(undef, 0)
        for j in eachindex(new_corr[k]) # ta and tb gevp index 
            _, data_meff = juobs.meff(new_corr[k][j], [tmin[end], tmax[1]], pl=true, data=true, wpm=wpm)
            _, _, param, _, aic = juobs.bayesian_av(one_exp_fit, data_meff, tmin, tmax, Nparam, pl=false, wpm=wpmm, data = true)
            close("all")
            AIC  = vcat(AIC, aic )
            Masses  = vcat(Masses, param)
        end
        isnothing(wpm) ? uwerr.(Masses) : [uwerr(Masses[i], wpm) for i in eachindex(Masses)]
        offset = minimum(AIC)
        AIC = AIC .- offset
        Wtot = exp.(-0.5 .* AIC)
        Wtot /= sum(Wtot)
        MAve[k] = sum(Masses .* Wtot) 
        isnothing(wpm) ? uwerr(MAve[k]) : uwerr(MAve[k], wpm)

        if pl
            
            subplot(211)
            title(string(L"$\mu_1 = $", corr[k]. mu[1], L" $\mu_2 = $", corr[k]. mu[2]))
            ax1 = gca()
            setp(ax1.get_xticklabels(),visible=false) # Disable x tick labels
            xx = collect(1:length(Masses))
            vv = value.(Masses)
            errorbar(xx, vv, err.(Masses), fmt="s")
            aux1 = sum( Wtot .* Masses.^2)
            aux2 = sum( Wtot .* Masses)^2
            syst = sqrt(abs(value(aux1-aux2)))
            fill_between(xx, value(MAve[k]) - err(MAve[k]), value(MAve[k]) + err(MAve[k]),color="red", alpha=0.2)
            ylabel(L"$M_0$")

            subplot(212)
            plot(xx, Wtot)
            ylabel(L"$W$")
            xlabel("Models")
            display(gcf())
            close()
        end
    end
    return MAve
end
function gevp_dec(mat_obs::Vector{MatInfo}, mass::Vector{uwreal}, path_plat::String; t0::Int64=2, wpm::Union{Nothing, Dict{Int64,Vector{Float64}}}=nothing, pl::Bool=true, n::Int64=1, pseudo::Bool=true, wilson::Bool=false)
    y0s       = getfield.(mat_obs, :y0)
    mu_list   = getfield.(mat_obs, :mu) 
    plat_dec0 = Vector{uwreal}(undef, length(mat_obs))
    data      = Vector{Vector{uwreal}}(undef,0)
    for i in 1:length(mat_obs)
        mat   = getfield(mat_obs[i], :mat_list)[y0s[i]+2:end-1] #matrices to diagonalise for a given sector [i]
        evecs = getall_eig(mat, t0)
        elem  = mat_elem(evecs, mat, mass[i], n) #ground and excited state energy
        push!(data, elem)
        plat  = select_plateau(mat_obs[i].ensinfo, mu_list, path_plat)[i] .- y0s[i]
        try
            if pseudo
                plat_dec0[i] =  dec(elem, plat, mass[i], mu_list[i], pl=pl, wilson=wilson)
            else
                plat_dec0[i] =  vec_dec(elem, plat, mass[i], pl=pl)
            end
        catch
            println("Decay constant for the ensemble ", mat_obs[i].ensinfo.id, " failed in the sector ", mu_list[i], " pseudo sector?", pseudo)
            println("The associated effective mass is ", mass[i])
            plat_dec0[i] = uwreal(0)
        end
    end
    return plat_dec0, data
end
function gevp_dec_BMA(mat_obs::Vector{MatInfo}, mass::Vector{uwreal}; t0::Int64=2, wpm::Union{Nothing, Dict{String,Vector{Float64}}}=nothing, pl::Bool=true, n::Int64=1, pseudo::Bool=true, wilson::Bool=false)
    y0s       = getfield.(mat_obs, :y0)
    mu_list   = getfield.(mat_obs, :mu) 
    plat_dec0 = Vector{uwreal}(undef, length(mat_obs))
    data      = Vector{Vector{uwreal}}(undef,0)
    for i in eachindex(mat_obs)
        mat   = getfield(mat_obs[i], :mat_list) #matrices to diagonalise for a given sector [i]
        evecs = getall_eig(mat, t0)
        elem  = mat_elem(evecs, mat, mass[i], n) #ground and excited state energy
        push!(data, elem)
        @. gs_model(x,p) = p[1] + p[2] * exp(-p[3]*x)
        param = 3
        tmin  =  collect(4:8) # scart from source 
        tmax  =  collect(40:2:50) 
        
        aux_plat_dec0, syst, fit_param, dict = juobs.bayesian_av(gs_model, elem, tmin, tmax, param, pl=pl, wpm=wpm, data=true, label=L"$f_{\mathrm{ps}}$")

        if pl
            uwerr.(elem)
            vv = value.(elem)
            ee = err.(elem)
            xx = collect(1:length(vv))
            errorbar(xx, vv, ee, fmt="*")
            ylim(vv[50] - 0.2, vv[50] + 0.2)
            display(gcf())
            close()
        end
        #try
            if pseudo
                plat_dec0[i] =  dec_automatic_plat(aux_plat_dec0, mass[i], mu_list[i], pl=pl, wilson=wilson)
            else
                plat_dec0[i] =  vec_dec(elem, plat, mass[i], pl=pl)
            end
        #catch
        #    println("Decay constant for the ensemble ", mat_obs[i].ensinfo.id, " failed in the sector ", mu_list[i], " pseudo sector?", pseudo)
        #    println("The associated effective mass is ", mass[i])
        #    plat_dec0[i] = uwreal(0)
        #end
    end
    return plat_dec0, data
end
function mat_elem(evec::Vector{Matrix{uwreal}}, ct_mat::Vector{Matrix{uwreal}}, mass::uwreal,  n::Int64)
    t = length(evec)
    res = Vector{uwreal}(undef, t)
    for i in 1:t
        aux = uwdot(evec[i][:,n], uwdot(ct_mat[i], evec[i][:,n]))[1]
        aux2 = 1 / (aux^2)^(0.25)   * exp(mass * (i)/2)
        res[i] = aux2 * uwdot(evec[i][:,n], ct_mat[i][:,n])
    end
    return res
end
function dec(mat_elem::Vector{uwreal}, plat::Array{Int64}, mass::uwreal, mu::Vector{Float64}; pl::Bool=true, wilson::Bool=false)
    aux = plat_av(mat_elem, plat)
    if pl
        uwerr(aux)
        x = 1:length(mat_elem)
        y = value.(mat_elem)
        dy = err.(mat_elem)
        v = value(aux)
        e = err(aux)
        figure()
        errorbar(x, y, yerr=dy, fmt="*", color="red")
        fill_between(plat[1]:plat[2], v-e, v+e, color="green")    
        ylim(v - 10*e, v + 10*e)
        display(gcf())
        close()
    end
    if !wilson
        return sqrt(2/ (mass)^3) * sum(mu)* aux
    else
        return sqrt(2/mass) * aux 
    end
end
function dec_automatic_plat(mat_elem::uwreal, mass::uwreal, mu::Vector{Float64}; pl::Bool=true, wilson::Bool=false)
    if !wilson
        return sqrt(2/ (mass)^3) * sum(mu)* mat_elem
    else
        return sqrt(2/mass) * mat_elem 
    end
end
function vec_dec(mat_elem::Vector{uwreal}, plat::Array{Int64}, mass::uwreal; pl::Bool=true)
    aux = plat_av(mat_elem, plat)
    if pl
        uwerr(aux)
        x = 1:length(mat_elem)
        y = value.(mat_elem)
        dy = err.(mat_elem)
        v = value(aux)
        e = err(aux)
        figure()
        ylim(v - 10*e, v + 10*e)
        errorbar(x, y, yerr=dy, fmt="*", color="red")
        fill_between(plat[1]:plat[2], v-e, v+e, color="green")    
        display(gcf())
    end
    return sqrt(2 / mass) * aux #aux / mass
end
function getall_eig(a::Vector{Matrix{uwreal}}, t0; iter=10)
    n = length(a)
    res = Vector{Matrix{uwreal}}(undef, n)
    [res[i] = uweigvecs(a[i], a[t0]) for i=1:n]
    return res 
end
function get_meff_tm(obs::Vector{juobs.Corr}, ens::EnsInfo; pl::Bool=false, wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing)
    mu_list  = getfield.(obs, :mu)
    plat = select_plateau_tm(ens, mu_list)
    println(plat)
    return meff.(obs, plat, pl=pl, wpm=wpm)
end
function get_meff_vec(obs::Vector{juobs.Corr}, ens::EnsInfo; pl::Bool=false, wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing)
    mu_list  = getfield.(obs, :mu)
    plat = select_plateau_vec(ens, mu_list)
    println(plat)
    return meff.(obs, plat, pl=pl, wpm=wpm)
end
function get_meff_w(obs::Vector{juobs.Corr}, ens::EnsInfo; pl::Bool=false, wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing)
    kappa_list  = getfield.(obs, :kappa)

    plat = select_plateau_w(ens, kappa_list)
    println(plat)
    return meff.(obs, plat, pl=pl, wpm=wpm)
end
function get_f_tm(obs::Vector{juobs.Corr}, m::Vector{uwreal}, ens::EnsInfo, path_plat::String; pl::Bool=false, wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing)
    mu_list = getfield.(obs, :mu)
    plat = select_plateau(ens, mu_list, path_plat)
    return dec_const_pcvc.(obs, plat, m, pl=pl, wpm=wpm)
end
function get_fstar_tm(obs::Vector{juobs.Corr}, m::Vector{uwreal}, ens::EnsInfo, path_plat::String; pl::Bool=false, wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing)
    mu_list = getfield.(obs, :mu)
    plat = select_plateau(ens, mu_list, path_plat)
    return dec_const_w.(obs, obs, plat, m, pl=pl, wpm=wpm)
end
"""
This function computes the decay constant for wilson twisted mass fermions and perform a 
    bayesian model average for plateau determination.
"""
function get_f_tm_BMA(corr::Vector{juobs.Corr}, m::Vector{uwreal}, ens::EnsInfo; pl::Bool=true, wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing, 
    path_plt::Union{String,Nothing}=nothing , ps::String="fps" )
   
    F = Vector{uwreal}(undef, length(corr))
    if !isnothing(path_plt) path_plt = joinpath(path_plt,"plateaus", ens.id, ps)  end
    @. const_fit(x,p) = p[1] #+ p[2]*exp(-p[3]*(x-corr[1].y0))
    
    tminaux = round.(Int64, collect(15:32) *  sqrt(value(t0(ens.beta) / t0(3.7))))
    T = length(corr[1].obs)
    tmaxaux = round.(Int64, collect(10:25) * sqrt(value(t0(ens.beta) / t0(3.7))))
    tmax = T .- collect(reverse(tmaxaux))
    tmax = tmax[1:3:end]
    tmin = tminaux[1:2:end] .+ corr[1].y0
    

    for k in eachindex(corr)
        # tmin = collect(14:30)    .+ens.y0
        # tmax = collect(38:4:60) .+ens.y0
        # if k==1
            # tmin .-=10
        # end
        t = string(L"$f \  \mu_1 = $", corr[k].mu[1],L" $\mu_2 = $", corr[k].mu[2] )
        plat = [tmin[end]-5,tmax[1]+5]
        if ps=="fps"
            _, data = dec_const_pcvc(corr[k], plat, m[k], pl=false, wpm=wpm, data=true)
        else
            _, data = dec_const(corr[k], plat, m[k], pl=false, wpm=wpm, data=true)
        end
        ll = ps == "fps" ? L"$f_{\mathrm{ps}}$" : L"$f_{\mathrm{v}}$"
        name_bma = "f$(k)_BMA.pdf"
        isnothing(path_plt) ? tt=nothing : tt = joinpath(path_plt,name_bma)
        try
            F[k], syst, all_res, FitDict = juobs.bayesian_av(const_fit, data, tmin, tmax, 1, pl=pl, wpm=wpm, data=true, path_plt=tt, plt_title=t, label=ll )
        
            if pl
                 
             
                try 
                    fig = figure(figsize=(10,7.5))
                    modstot = FitDict["mods"]
                    AIC  = FitDict["AIC"]
                    Wtot = FitDict["W"]
                    IDX  = partialsortperm(AIC, 1:3)
                    mods = modstot[IDX]
                    main_res = all_res[IDX]
                    W = Wtot[IDX]
                    isnothing(wpm) ? uwerr(data) : [uwerr(data[i],wpm) for i in eachindex(data)]
                    vv = value.(data)[corr[1].y0:end]
                    ee = err.(data)[corr[1].y0:end]
                    xx = collect(0:length(vv)-1) .+ corr[1].y0
                    errorbar(xx, vv, ee, fmt="d", mfc="none", color="black", capsize=2)
                    CC = ["forestgreen", "royalblue", "tomato"]
                    for (ii,trange) in enumerate(mods)
                        isnothing(wpm) ? uwerr(main_res[ii]) : uwerr(main_res[ii], wpm)
                        vaux = value(main_res[ii])
                        eaux = err(main_res[ii])
                        regex = r"([0-9]+),([0-9]+)"
                        aux = match(regex, trange)
                        fill_between(parse(Float64,aux[1]):parse(Float64,aux[2]), vaux-eaux, vaux+eaux, color=CC[ii], alpha=0.5, label=round(W[ii], digits=3))
                    end

                    axhline(y=value(main_res[1])-14*err(main_res[1]), xmin=(tmin[1]-corr[1].y0)/length(xx), xmax=(tmin[end]-corr[1].y0)/length(xx), ls="dashed", color="green")
                    axhline(y=value(main_res[1])-14*err(main_res[1]), xmin=(tmax[1]-corr[1].y0)/length(xx), xmax=(tmax[end]-corr[1].y0)/length(xx), ls="dashed", color="red")
                    text(tmin[1],value(main_res[1])-13*err(main_res[1]), L"$\{t_{\mathrm{min}}\}$" )
                    text(tmax[1],value(main_res[1])-13*err(main_res[1]), L"$\{t_{\mathrm{max}}\}$" )


                    # axvline(tmin[1], ls="dashed", color="green")
                    # axvline(tmax[end], ls="dashed", color="red")
                    legend(title="Weights")
                    title(t)
                    xlabel(L"$t/a$")
                    ylabel(ll)
                    xlim(xx[1], xx[end])
                    ylim(value(main_res[1]) -20*err(main_res[1]), value(main_res[1]) +20*err(main_res[1]))
                    twiny()
                    xlim(xx[1], xx[end])
                    fm_time = round.((xx .- corr[1].y0)[1:8:end] .* value(a(ens.beta)), digits=3)
                    xticks(xx[1:8:end], fm_time)
                    # xticks()
                    xlabel(L"$t-t_0\ [\mathrm{fm}]$", fontsize=14)
                    display(fig)
                    if !isnothing(path_plt)
                        savefig(joinpath(path_plt, "f$(k)_plateau.pdf"))
                    end
                    close()
                catch
                   @warn "plot failed"
                end
            end
        catch
            @warn "Decay failed"
            F[k] = uwreal(0.0)
        end
   
    end
    return F
end
function select_plateau(ensinf::EnsInfo, mu_list, path_plat)
    ens =ensinf.id
    deg = ensinf.deg
    mul, mus, muh = get_mu(mu_list, deg)
    f = readdlm(path_plat)
    head = String.(f[:, 1])

    delim = findall(x-> occursin("#", x ), head)
    edelim = findfirst(x-> occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)

    del = delim[cdelim]+1 : delim[cdelim+1]-1
    #del = delim
    plat = Vector{Vector{Int64}}(undef, 0)
    #plat = Dict()
    plat_ = Int64.(f[del, 2:3])
    head_ = String.(f[del, 1])
    
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] != mu[2] && !(mus in mu)  #heavy-light
            n = findfirst(x-> occursin("lh", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]]  ) 
        elseif mul in mu && mu[1] == mu[2] #light-light
            n = findfirst(x-> occursin("ll", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )               
        elseif mul in mu && mus in mu#strange-light
            n = findfirst(x-> occursin("ls", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] != mu[2] && !(mul in mu)#heavy-strange
            n = findfirst(x-> occursin("sh", x ), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] == mu[2] && !(mul in mu)#strange-strange
            n = findfirst(x-> occursin("ss", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif !(mul in mu) && !(mus in mu) #heavy-heavy
            n = findfirst(x-> occursin("hh", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        end
    end
    return plat
end
function select_plateau_t0(ensinfo::EnsInfo)
    ens = ensinfo.id
    f = readdlm(path_plat_w)
    head  = String.(f[:,1])
    delim = findall(x->occursin("#", x), head)
    edelim = findfirst(x->occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)
    del = delim[cdelim]+1 : delim[cdelim+1]-1
    head_ = String.(f[del, 1])
    plat_ = Int64.(f[del, 2:3])
    plat = Vector{Int64}(undef, 2)
    n = findfirst(x-> occursin("t0", x), head_)
    plat[1] = plat_[n,1]
    plat[2] = plat_[n,2]
    return plat
end
function select_plateau_tm(ensinf::EnsInfo, mu_list)
    ens =ensinf.id
    deg = ensinf.deg
    mul, mus, muh = get_mu(mu_list, deg)
    f = readdlm(path_plat_tm)
    head = String.(f[:, 1])

    delim = findall(x-> occursin("#", x ), head)
    edelim = findfirst(x-> occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)

    del = delim[cdelim]+1 : delim[cdelim+1]-1
    #del = delim
    plat = Vector{Vector{Int64}}(undef, 0)
    #plat = Dict()
    plat_ = Int64.(f[del, 2:3])
    head_ = String.(f[del, 1])
    
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] != mu[2] && !(mus in mu)  #heavy-light
            n = findfirst(x-> occursin("lh", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]]  ) 
        elseif mul in mu && mu[1] == mu[2] #light-light
            n = findfirst(x-> occursin("ll", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )               
        elseif mul in mu && mus in mu#strange-light
            n = findfirst(x-> occursin("ls", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] != mu[2] && !(mul in mu)#heavy-strange
            n = findfirst(x-> occursin("sh", x ), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] == mu[2] && !(mul in mu)#strange-strange
            n = findfirst(x-> occursin("ss", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif !(mul in mu) && !(mus in mu) #heavy-heavy
            n = findfirst(x-> occursin("hh", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        end
    end
    return plat
end
function (ensinf::EnsInfo, mu_list)
    ens =ensinf.id
    deg = ensinf.deg
    mul, mus, muh = get_mu(mu_list, deg)
    f = readdlm(path_plat_tm)
    head = String.(f[:, 1])

    delim = findall(x-> occursin("#", x ), head)
    edelim = findfirst(x-> occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)

    del = delim[cdelim]+1 : delim[cdelim+1]-1
    #del = delim
    plat = Vector{Vector{Int64}}(undef, 0)
    #plat = Dict()
    plat_ = Int64.(f[del, 2:3])
    head_ = String.(f[del, 1])
    
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] != mu[2] && !(mus in mu)  #heavy-light
            n = findfirst(x-> occursin("lh_vec", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]]  ) 
        elseif mul in mu && mu[1] == mu[2] #light-light
            n = findfirst(x-> occursin("ll_vec", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )               
        elseif mul in mu && mus in mu#strange-light
            n = findfirst(x-> occursin("ls_vec", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] != mu[2] && !(mul in mu)#heavy-strange
            n = findfirst(x-> occursin("sh_vec", x ), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] == mu[2] && !(mul in mu)#strange-strange
            n = findfirst(x-> occursin("ss_vec", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif !(mul in mu) && !(mus in mu) #heavy-heavy
            n = findfirst(x-> occursin("hh_vec", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        end
    end
    return plat
end
function select_plateau_w(ensinf::EnsInfo, kappa_list)
    ens = ensinf.id
    deg = ensinf.deg
    kappal, kappas = get_kappa(kappa_list, deg)
    f = readdlm(path_plat_w)
    head = String.(f[:, 1])

    delim = findall(x-> occursin("#", x ), head)
    edelim = findfirst(x-> occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)

    del = delim[cdelim]+1 : delim[cdelim+1]-1
    #del = delim
    plat = Vector{Vector{Int64}}(undef, 0)
    #plat = Dict()
    plat_ = Int64.(f[del, 2:3])
    head_ = String.(f[del, 1])
    for k = 1:length(kappa_list)
        kappa = kappa_list[k]
        if kappal in kappa && kappa[1] == kappa[2] #light-light
            n = findfirst(x-> occursin("ll", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )               
        elseif kappal in kappa && kappas in kappa#strange-light
            n = findfirst(x-> occursin("ls", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )

        elseif kappas in kappa && kappa[1] == kappa[2] && !(kappal in kappa)#strange-strange
            n = findfirst(x-> occursin("ss", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        end
    end
    return plat
end


function match_muc(muh, m_lh, m_sh, target; wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing)
    M = (2/3 .* m_lh .+ 1/3 .* m_sh) 
    par, chi2exp = lin_fit(muh, M, wpm=wpm)
    muh_target = x_lin_fit(par, target)
    return muh_target
end

function aic_weight(aic)
    w_aux = exp.(-1/2 * aic)  
    w_aux ./= sum(w_aux)
    idx_in = sortperm(w_aux, rev=true)
    cum_w = cumsum(w_aux[idx_in])
    #stop = findfirst(x->x>=0.999, cum_w)
    stop = length(aic)
    idx = sort(idx_in[1:stop])
    w = w_aux[idx] ./ sum(w_aux[idx])
    # w = w_aux ./ sum(w_aux)
    return w , idx# idx_in[1:stop]
end
aic_weight(c::Cat) = aic_weight(c.aic)
# function aic_weight(c::Cat)
    # aic = c.aic
    # w_aux = exp.(-1/2 * aic)  
    # w_aux = exp.(-1/2 * (aic .- minimum(aic)))  
    # w_aux = exp.(-1/2 * (aic ./ minimum(aic))) #  used at LAT22
    # w_aux1=filter(x->x<10000, w_aux)
    # index = findall(x->x<10000, w_aux)
    # norm = sum(w_aux1)
    # w = w_aux1 ./ norm
    # return w, index
# end
function aic_syst(aic, fit_res)
    w,index = aic_weight(aic)
    aux1 = sum( w .* fit_res[index].^2)
    aux2 = sum( w .* fit_res[index])^2
    return sqrt(abs(value(aux1 - aux2)))
end
aic_syst(c::Cat) = aic_syst(c.aic, c.fit_res)
# function aic_syst(c::Cat) = aic_syst(c.aic. c.fit_res)
    # w,index = aic_weight(c.aic)
    # aux1 = sum( w .* c.fit_res[index].^2)
    # aux2 = sum( w .* c.fit_res[index])^2
    # return sqrt(abs(value(aux1 - aux2)))
# end

function aic_model_ave(aic, fit_res, wpm::Union{Nothing, Dict}=nothing)
    w, index = aic_weight(aic)
    res = sum( w.* fit_res[index])
    isnothing(wpm) ? uwerr(res) : uwerr(res, wpm)
    return res 
end
aic_model_ave(c::Cat, wpm::Union{Nothing,Dict}=nothing) = aic_model_ave(c.aic, c.fit_res, wpm)
# function aic_model_ave(c::Cat, wpm::Union{Nothing, Dict}=nothing)
    # w, index = aic_weight(c)
    # res = sum( w.* getfield(c, :fit_res)[index])
    # isnothing(wpm) ? uwerr(res) : uwerr(res, wpm)
    # return res 
# end
function category_av(cat_arr::Vector{Cat})
    best_res = aic_model_ave.(cat_arr)
    syst = aic_syst.(cat_arr)
    tot_err = err.(best_res).^2 .+ syst.^2
    w_aux = 1 ./ (tot_err)
    w = w_aux ./ sum(w_aux)
    println(findmax(w)[2])
    aux1 = sum( w .* best_res.^2)
    aux2 = sum( w .* best_res)^2
    sys1 = sqrt(abs(value(aux1 - aux2)))
    av = sum( w .* best_res)

    return av, sys1
end
function category_av_tot(cat_arr::Vector{Cat})
    fit_res = vcat(getfield.(cat_arr, :fit_res)...)
    aic = vcat(getfield.(cat_arr, :aic)...)
    best = aic_model_ave(aic, fit_res)
    syst = aic_syst(aic, fit_res)
    return best, syst
end

function syst_av(syst_err::Vector{Float64})
    aux = sum(syst_err.^2)
    return sqrt(aux)
end

function cat_obs(obs::Vector{Vector{uwreal}})
    N = length(obs)
    if !all(length.(obs) .== N)
        error("Dimension mismatch")
    end

    r = getindex.(obs, 1)
    for i = 2:N
        r = vcat(r, getindex.(obs, i))
    end
    return r
end
function cat_obs_vcat(obs::Vector{Vector{uwreal}})
    return vcat(obs...)
end


function fit_over_cat!(cat_tot::Vector{Cat}, models::Vector{Vector{Function}}, phi2_phys::uwreal, model_param::Vector{Int64} ; decay::Bool=false, 
    weight_dof::Int64=2, xycovar::Bool=false, correlated_fit::Bool=false, Mrat::uwreal=uwreal(1.0),
    wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing, IC::String="AIC_small_sample")
    
    models  = vcat(models...)

    function continuum_dependece(params, phi2, phih; decay_phih_dep::Bool=decay)
        if !decay_phih_dep
            return Mrat * (params[1] + params[2]*phi2 + params[3]*phih)
        else
            return params[1] + params[2]*phi2 + params[3]/sqrt(phih)
        end
    end

    for cat in cat_tot
        println("\n", cat.info, "\n")
        for (i, mod) in enumerate(models)
            if !xycovar # correlation between x and y 
                if !correlated_fit
                    fit_params, chi2, chi2exp = juobs.fit_routine(mod, value.(cat.x_tofit), cat.obs, model_param[i], wpm=wpm)
                else
                    cov_inv = juobs.inv_covar_multi_id(cat.obs)
                    fit_params, chi2, chi2exp = juobs.fit_routine(mod, value.(cat.x_tofit), cat.obs, model_param[i], inv_cov=cov_inv, wpm=wpm)
                end
            else
                fit_params, chi2, chi2exp = juobs.fit_routine(mod, cat.x_tofit, cat.obs, model_param[i], covar=true, wpm=wpm)                
            end

            val_mod_i = continuum_dependece(fit_params, phi2_phys, cat.phih_ph)
            push!(cat.dof, length(cat.obs)-model_param[i] ) 
            if IC == "AIC_small_sample"
                ic = chi2 / chi2exp * cat.dof[i] + weight_dof * model_param[i] + ( weight_dof*model_param[i]^2 + weight_dof*model_param[i]) / (length(cat.obs)- model_param[i] -1)
            elseif IC == "AIC"
                ic =  chi2 + 2*model_param[i] #chi2 / chi2exp * cat.dof[i] + weight_dof * model_param[i]                
            elseif IC == "TIC"
                ic = chi2  - 2*chi2exp 
            end
            # 
            # if !small_sample
                # AIC = chi2_vs_chi2exp * cat.dof[i] + weight_dof * model_param[i]
            # else
                # AIC = chi2_vs_chi2exp * cat.dof[i] + weight_dof * model_param[i] + ( weight_dof*model_param[i]^2 + weight_dof*model_param[i]) / (length(cat.obs)- model_param[i] -1)
            # end
            tmp  = [val_mod_i, fit_params, chi2/chi2exp, model_param[i], ic]
            symb = [:fit_res, :fit_param, :chi2_vs_exp, :n_param, :aic]
            for (k,ss) in enumerate(symb)
                push!(getfield(cat, ss), tmp[k] )
            end
        end
    end
end

function Base.:*(x::uwreal, y::Array{Any})
    N = size(y, 1)
    return fill(x, N) .* y
end  
Base.:*(x::Array{Any}, y::uwreal) = Base.:*(y,x)
function Base.:*(x::uwreal, y::Array{Float64})
    N = size(y, 1)
    return fill(x, N) .* y
end
Base.:*(x::Array{Float64}, y::uwreal) = Base.:*(y,x)
function Base.:*(x::uwreal, y::Array{uwreal})
    N = size(y, 1)
    return fill(x, N) .* y
end
Base.:*(x::Array{uwreal}, y::uwreal) = Base.:*(y,x)
function Base.:+(x::uwreal, y::Vector{uwreal})
    N = size(y, 1)
    return fill(x, N) .+ y
end
function Base.:+(x::Float64, y::Vector{Float64})
    N = size(y, 1)
    return fill(x, N) .+ y
end
function Base.:+(x::Float64, y::Vector{uwreal})
    N = size(y, 1)
    return fill(x, N) .+ y
end
function Base.:-(x::Float64, y::Vector{Any})
    N = size(y, 1)
    return fill(x, N) .- y
end
function Base.:-(x::Float64, y::Vector{Float64})
    N = size(y, 1)
    return fill(x, N) .- y
end
function Base.:-(x::Float64, y::Vector{uwreal})
    N = size(y, 1)
    return fill(x, N) .- y
end